# Copyright (c) 2010, Cloudera, inc.
# All rights reserved.

ROOT ?= ../../../
ROOT := $(realpath $(ROOT))

ifneq ($(shell test -f $(ROOT)/Makefile.vars && echo $$?),0)
  $(error "Error: Please set $$ROOT to point to the directory containing Makefile.vars.  (Currently: $(ROOT))")
endif

include $(ROOT)/Makefile.vars

ENV ?= $(BLD_DIR_ENV)
ENV_PYTHON ?= $(ENV)/bin/python
ENV_EASY_INSTALL ?= $(ENV)/bin/easy_install

APP_ROOT := $(realpath .)
APP_NAME ?= $(notdir $(APP_ROOT))
PKG_NAME ?= APP_NAME
FLAVOR ?=
ifeq ($(FLAVOR),)
  DASH_FLAVOR :=
else
  DASH_FLAVOR := -$(FLAVOR)
endif

APP_VERSION = $(shell $(ENV_PYTHON) setup.py -V)

EXT_PY_DIRS := $(wildcard ext-py/*)
EXT_PYS := $(notdir $(EXT_PY_DIRS))
BUILD_DIR := $(APP_ROOT)/build/
BDIST_PARENT_DIR := $(BUILD_DIR)/bdist
BDIST_DIR := $(BDIST_PARENT_DIR)/$(APP_NAME)$(DASH_FLAVOR)
SDIST_PARENT_DIR := $(BUILD_DIR)/sdist
SDIST_DIR := $(SDIST_PARENT_DIR)/$(APP_NAME)$(DASH_FLAVOR)
APP_TGZ=$(APP_NAME)$(DASH_FLAVOR)-$(APP_VERSION).tar.gz

SRCDEB_DIR := $(BUILD_DIR)/srcdeb
RPM_TOPDIR := $(BUILD_DIR)/rpm
RPM_SPEC_FILE ?= $(APP_ROOT)/$(APP_NAME).spec


##############################
# Macros
##############################

ext_py_stamp = $(BUILD_DIR)/$(1)/$(2).stamp

##############################
# Top level targets
##############################

default:: build
.PHONY: default

# Hook to allow extra build steps (eg java builds)
build:
.PHONY: build

clean:: thirdparty-clean
	$(PYTHON) setup.py clean || :
	@rm -Rf $(BUILD_DIR)
	@find . -name \*.egg-info -prune -exec rm -Rf {} \;
	@find . -name \*.py[co] -exec rm -f {} \;
.PHONY: clean

# Install the application into the virtual environment.
env-install: build thirdparty-env-install
	$(ENV_PYTHON) setup.py develop -N
.PHONY: env-install

# Uninstall the application from the virtual environment
# This does not uninstall the third party, since other apps
# may depend on the same thirdparty dependencies
env-uninstall:
	$(ENV_PYTHON) setup.py develop -u
.PHONY: env-uninstall

# Install the binary application into a target directory
# This does *not* install it into an environment.
install-binary-dist: binary-dist
	@echo "--- Install binary distribution at $(INSTALL_APPS_DIR)/$(APP_NAME) ---"
	@mkdir -p $(INSTALL_APPS_DIR)/$(APP_NAME)
	@rsync -a $(BDIST_DIR)/ $(INSTALL_APPS_DIR)/$(APP_NAME)
ifneq ($(wildcard conf),) # if there are conf files
	@echo "--- Installing configuration into $(INSTALL_DIR)/desktop/conf ---"
	@mkdir -p $(INSTALL_DIR)/desktop/conf
	@for conffile in $(notdir $(wildcard $(BDIST_DIR)/conf/*)) ; do \
	  if [ -f $(INSTALL_DIR)/desktop/conf/$$conffile ]; then \
	    echo Moving aside old config $(INSTALL_DIR)/desktop/conf/$$conffile ; \
	    mv $(INSTALL_DIR)/desktop/conf/$$conffile \
	          $(INSTALL_DIR)/desktop/conf/$$conffile.save.$(shell date +"%Y%m%d.%H%M%S") ; \
	  fi ; \
	  cp $(BDIST_DIR)/conf/$$conffile $(INSTALL_DIR)/desktop/conf/$$conffile ; \
	done
	@# Remove the conf dir from the install location since we've placed the confs in the
	@# right spot
	@ rm -Rf $(INSTALL_APPS_DIR)/$(APP_NAME)/conf
endif

.PHONY: install-binary-dist

distclean: clean
	@rm -Rf $(APP_DIST_DIR)
	@rm -Rf $(BDIST_DIR)
.PHONY: distclean



##############################
# Third-party targets
#
# These macros generate thirdparty-egg, thirdparty-clean, and
# thirdparty-env-install targets
##############################
EXT_PY_EGG_TARGETS := $(EXT_PYS:%=$(call ext_py_stamp,%,egg))
EXT_PY_CLEAN_TARGETS := $(EXT_PYS:%=$(call ext_py_stamp,%,clean))
EXT_PY_ENV_INSTALL_TARGETS := $(EXT_PYS:%=$(call ext_py_stamp,%,env-install))

$(EXT_PY_EGG_TARGETS): CUR_EXT_PY=$(notdir $(@D))
$(EXT_PY_CLEAN_TARGETS): CUR_EXT_PY=$(notdir $(@D))
$(EXT_PY_ENV_INSTALL_TARGETS): CUR_EXT_PY=$(notdir $(@D))


# thirdparty-egg
thirdparty-egg: $(EXT_PY_EGG_TARGETS)
$(EXT_PY_EGG_TARGETS):
	@echo --- Building $(CUR_EXT_PY)...
	@mkdir -p $(BUILD_DIR)/$(CUR_EXT_PY)
	@# This nasty command builds a binary egg even for packages that are
	@# based on the earlier distutils (rather than setuptools).
	@# TODO(todd) refactor this to a script, maybe? My perl background
	@# is showing through...
	@cd ext-py/$(CUR_EXT_PY) && $(ENV_PYTHON) \
	  -c '__import__("setuptools.sandbox").sandbox.run_setup("setup.py", __import__("sys").argv[1:])' \
	  bdist_egg
	@touch $@

# thirdparty-clean
thirdparty-clean: $(EXT_PY_CLEAN_TARGETS)
$(EXT_PY_CLEAN_TARGETS):
	@echo --- Cleaning $(CUR_EXT_PY)...
	@rm -Rf $(BUILD_DIR)/$(CUR_EXT_PY)
	@cd ext-py/$(CUR_EXT_PY) && rm -Rf dist build temp *.egg-info
	@find ext-py/$(CUR_EXT_PY) -name \*.egg-info -o -name \*.py[co] -prune -exec rm -Rf {} \;

.SECONDEXPANSION:
thirdparty-env-install: $(EXT_PY_ENV_INSTALL_TARGETS)
$(EXT_PY_ENV_INSTALL_TARGETS): $(call ext_py_stamp,$$(CUR_EXT_PY),egg)
	@echo --- Installing $(CUR_EXT_PY) into virtual environment...
	cd ext-py/$(CUR_EXT_PY) && $(ENV_EASY_INSTALL) -N dist/*egg 2>/dev/null


.PHONY: thirdparty-egg thirdparty-clean thirdparty-env-install
############################################################
# Sub targets
############################################################

# Items that should not be included in the source/binary distribution
# Apps can list this in their own Makefile before including
# app.mk in order to exclude their own sources, non-distributables,
# etc.
COMMON_EXCLUDES := \
	  --exclude=build \
	  --exclude=\*.py[co] \
	  --exclude=\*.sw[op] \
	  --exclude=~\* \
	  --exclude=.gitignore

SDIST_RSYNC_EXCLUDES += $(COMMON_EXCLUDES)
BDIST_RSYNC_EXCLUDES += $(COMMON_EXCLUDES) --exclude=ext-py

# Make a new tree for this app with only the binary eggs
.PHONY: binary-dist
binary-dist: build thirdparty-egg
	@# Dist should be clean
	@mkdir -p $(BDIST_DIR)

	@# Copy sources of our app
	rsync -a ./ $(BDIST_DIR)/ \
	  $(BDIST_RSYNC_EXCLUDES)

	@# Copy built thirdparty dependencies
	if [ -n "$(EXT_PY_DIRS)" ] ; then \
	  mkdir -p $(BDIST_DIR)/ext-py-eggs ; \
	  cp -a $(foreach epd,$(EXT_PY_DIRS),$(wildcard $(epd)/dist/*egg)) $(BDIST_DIR)/ext-py-eggs/ ; \
	fi

.PHONY: src-dist
src-dist:
	@mkdir -p $(SDIST_DIR)
	@# Copy sources of our app
	rsync -a ./ $(SDIST_DIR)/ $(SDIST_RSYNC_EXCLUDES)


# Install thirdparty dependencies
thirdparty-env-install: thirdparty-egg
	@# Install any eggs that are already built in our dir
	if [ -n "$(wildcard ext-py-eggs/*)" ]; then \
	  $(ENV_EASY_INSTALL) -N $(wildcard ext-py-eggs/*.egg) 2>/dev/null ; \
	fi
.PHONY: thirdparty-env-install

##############################
# Per-app packaging targets:
#
# These targets are meant to help you package your application
# separately from Desktop Core. They are currently somewhat
# experimental and may change in later desktop releases.
#
# For the srcdeb and srpm targets, you must include a debian/
# directory and a .spec file within your application directory.
#
#  tar - make a tar of the binary distribution
#  src
#
#
##############################
tar: $(SDIST_PARENT_DIR)/$(APP_TGZ)

$(SDIST_PARENT_DIR)/$(APP_TGZ): src-dist
	cd $(SDIST_PARENT_DIR) && tar czf $(APP_TGZ) $(APP_NAME)$(DASH_FLAVOR)

srcdeb: tar
	@echo "--- Making src deb in $(SRCDEB_DIR) ---"
	@mkdir -p $(SRCDEB_DIR)
	@cp $(SDIST_PARENT_DIR)/$(APP_TGZ) \
		$(SRCDEB_DIR)/$(PKG_NAME)$(DASH_FLAVOR)_$(APP_VERSION).orig.tar.gz
	@tar -C $(SRCDEB_DIR) -zxf $(SRCDEB_DIR)/$(PKG_NAME)$(DASH_FLAVOR)_$(APP_VERSION).orig.tar.gz
	@cd $(SRCDEB_DIR)/$(APP_NAME)$(DASH_FLAVOR) && debuild -eDESKTOP_DIR=$(ROOT) -uc -us -S

srpm: tar
	@echo "--- Making srpm in $(RPM_TOPDIR) ---"
	@mkdir -p $(RPM_TOPDIR)
	@cd $(RPM_TOPDIR) && mkdir -p SRPMS SPECS SOURCES BUILD RPMS buildroot
	@cp $(SDIST_PARENT_DIR)/$(APP_TGZ) $(RPM_TOPDIR)/SOURCES/
	@rpmbuild --define "_topdir $(RPM_TOPDIR)" --buildroot=$(RPM_TOPDIR)/buildroot --nodeps -bs $(RPM_SPEC_FILE)
