#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Settings to configure the Health app.

Currently, this includes all ganglia clusters.  HDFS and MR
cluster configuration is picked up from hadoop.conf.
"""

from desktop.lib.conf import ConfigSection, UnspecifiedConfigSection, Config

GANGLIA_CLUSTERS = UnspecifiedConfigSection(
  "ganglia_clusters",
  help="One entry for each ganglia metad",
  each=ConfigSection(
    help="Information about a single Ganglia cluster",
    members=dict(
      GMETAD_HOST=Config("gmetad_host", help="Hostname/IP for the gmetad daemon."),
      GMETAD_PORT=Config("gmetad_port", help="Port for gmetad", default=8649, type=int),
      GANGLIA_URL=Config("ganglia_url", help="Url of Ganglia's frontend. (e.g., http://gmeta-host:80/ganglia/)"),
    )
  )
)

HEALTHD = ConfigSection(
  "healthd",
  help="Configuration for the health daemon",
  members=dict(
    HTTP_PORT=Config("http_port", help="Port for the healthd HTTP endpoint", default=6473, type=int),
    MEMBERSHIP_REFRESH_INTERVAL=Config(
      "membership_refresh_interval",
      help=("Healthd will refresh its view of the cluster membership " +
            "periodically. This setting determines the interval (in seconds)" +
            " at which it does so."),
      default=30,
      type=int),
    METRICS_FETCH_TIMEOUT=Config(
      "metrics_fetch_timeout",
      help=("Number of seconds to wait for HTTP response when fetching metrics " +
            "from Hadoop web UIs."),
      default=10,
      type=int),
    METRICS_FETCH_INTERVAL=Config(
      "metrics_fetch_interval",
      help=("Interval at which metrics are fetched from each host in the cluster. " +
            "This setting actually determines the amount of time between the " +
            "end of one fetch and the triggering of the next."),
      default=15,
      type=int),
    ))


# Various thresholds for lint
LINT_OVERRIDES = UnspecifiedConfigSection(
  "lint_checks",
  help="Overrides for lint check classes",
  each=ConfigSection(
    members=dict(
      warn_level=Config("warn_level", type=float),
      critical_level=Config("critical_level", type=float))))
