#!/usr/bin/python
# -*- coding: utf-8 -*-
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
# Some test cases for the Ganglia XML parser

import unittest
from ganglia_util import GangliaProxy
import StringIO

class TestGangliaProxy(unittest.TestCase):
  def test_repeated_parsing(self):
    proxy = GangliaProxy("localhost",0,url="http://fake-gangliaweb/")
    for i in xrange(2):
      file = StringIO.StringIO(TestGangliaProxy.ganglia_31_xml)
      proxy.parse(file)

  def test_31_parser(self):
    proxy = GangliaProxy("localhost",0,url="http://fake-gangliaweb/")
    file = StringIO.StringIO(TestGangliaProxy.ganglia_31_xml)
    proxy.parse(file)
    self.assertEqual(proxy.values["192.168.88.156"]["load_five"]["VAL"],
                     "0.21")
    self.assertEqual(len(proxy.values.keys()),1)
    self.assertEqual(len(proxy.values["192.168.88.156"].keys()),6)
    self.assertEqual(proxy.values["192.168.88.156"]["load_five"]["URL"],
                     "http://fake-gangliaweb/?&c=unspecified&h=192.168.88.156" +
                     "&m=load_five&r=hour&jr=&js=&vl=+&ti=Five+Minute+Load+Average&v=0.21")

  def test_30_parser(self):
    proxy = GangliaProxy("localhost",0,url="http://fake-gangliaweb/")
    file = StringIO.StringIO(TestGangliaProxy.ganglia_30_xml)
    proxy.parse(file)
    self.assertEqual(len(proxy.values.keys()),1)
    self.assertEqual(len(proxy.values["10.1.11.18"].keys()),32)
    self.assertEqual(proxy.values["10.1.11.18"]["disk_free"]["VAL"],"3.193")
    self.assertEqual(proxy.values["10.1.11.18"]["disk_free"]["URL"],
                     "http://fake-gangliaweb/?&c=customerb&h=10.1.11.18&m=disk_free" +
                     "&r=hour&jr=&js=&vl=GB&ti=&v=3.193")


  ganglia_30_xml = """<?xml version="1.0" encoding="ISO-8859-1" standalone="yes"?>
<!DOCTYPE GANGLIA_XML [
   <!ELEMENT GANGLIA_XML (GRID|CLUSTER|HOST)*>
      <!ATTLIST GANGLIA_XML VERSION CDATA #REQUIRED>
      <!ATTLIST GANGLIA_XML SOURCE CDATA #REQUIRED>
   <!ELEMENT GRID (CLUSTER | GRID | HOSTS | METRICS)*>
      <!ATTLIST GRID NAME CDATA #REQUIRED>
      <!ATTLIST GRID AUTHORITY CDATA #REQUIRED>
      <!ATTLIST GRID LOCALTIME CDATA #IMPLIED>
   <!ELEMENT CLUSTER (HOST | HOSTS | METRICS)*>
      <!ATTLIST CLUSTER NAME CDATA #REQUIRED>
      <!ATTLIST CLUSTER OWNER CDATA #IMPLIED>
      <!ATTLIST CLUSTER LATLONG CDATA #IMPLIED>
      <!ATTLIST CLUSTER URL CDATA #IMPLIED>
      <!ATTLIST CLUSTER LOCALTIME CDATA #REQUIRED>
   <!ELEMENT HOST (METRIC)*>
      <!ATTLIST HOST NAME CDATA #REQUIRED>
      <!ATTLIST HOST IP CDATA #REQUIRED>
      <!ATTLIST HOST LOCATION CDATA #IMPLIED>
      <!ATTLIST HOST REPORTED CDATA #REQUIRED>
      <!ATTLIST HOST TN CDATA #IMPLIED>
      <!ATTLIST HOST TMAX CDATA #IMPLIED>
      <!ATTLIST HOST DMAX CDATA #IMPLIED>
      <!ATTLIST HOST GMOND_STARTED CDATA #IMPLIED>
   <!ELEMENT METRIC EMPTY>
      <!ATTLIST METRIC NAME CDATA #REQUIRED>
      <!ATTLIST METRIC VAL CDATA #REQUIRED>
      <!ATTLIST METRIC TYPE (string | int8 | uint8 | int16 | uint16 | int32 | uint32 | float | double | timestamp) #REQUIRED>
      <!ATTLIST METRIC UNITS CDATA #IMPLIED>
      <!ATTLIST METRIC TN CDATA #IMPLIED>
      <!ATTLIST METRIC TMAX CDATA #IMPLIED>
      <!ATTLIST METRIC DMAX CDATA #IMPLIED>
      <!ATTLIST METRIC SLOPE (zero | positive | negative | both | unspecified) #IMPLIED>
      <!ATTLIST METRIC SOURCE (gmond | gmetric) #REQUIRED>
   <!ELEMENT HOSTS EMPTY>
      <!ATTLIST HOSTS UP CDATA #REQUIRED>
      <!ATTLIST HOSTS DOWN CDATA #REQUIRED>
      <!ATTLIST HOSTS SOURCE (gmond | gmetric | gmetad) #REQUIRED>
   <!ELEMENT METRICS EMPTY>
      <!ATTLIST METRICS NAME CDATA #REQUIRED>
      <!ATTLIST METRICS SUM CDATA #REQUIRED>
      <!ATTLIST METRICS NUM CDATA #REQUIRED>
      <!ATTLIST METRICS TYPE (string | int8 | uint8 | int16 | uint16 | int32 | uint32 | float | double | timestamp) #REQUIRED>
      <!ATTLIST METRICS UNITS CDATA #IMPLIED>
      <!ATTLIST METRICS SLOPE (zero | positive | negative | both | unspecified) #IMPLIED>
      <!ATTLIST METRICS SOURCE (gmond | gmetric) #REQUIRED>
]>
<GANGLIA_XML VERSION="3.0.7" SOURCE="gmetad">
<GRID NAME="CCS Grid" AUTHORITY="http://sea1-001-09.sl.cloudera.com/ganglia/" LOCALTIME="1249317650">
<CLUSTER NAME="customerb" LOCALTIME="1243386996" OWNER="Cloudera" LATLONG="unspecified" URL="http://www.cloudera.com/">
<HOST NAME="10.1.11.18" IP="10.1.11.18" REPORTED="1235604488" TN="13713161" TMAX="20" DMAX="0" LOCATION="unspecified" GMOND_STARTED="1235603937">
<METRIC NAME="machine_type" VAL="x86_64" TYPE="string" UNITS="" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="disk_free" VAL="3.193" TYPE="double" UNITS="GB" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="bytes_out" VAL="45.55" TYPE="float" UNITS="bytes/sec" TN="13713401" TMAX="300" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="gexec" VAL="OFF" TYPE="string" UNITS="" TN="13713401" TMAX="300" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="proc_total" VAL="171" TYPE="uint32" UNITS=" " TN="13713221" TMAX="950" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="pkts_in" VAL="0.52" TYPE="float" UNITS="packets/sec" TN="13713401" TMAX="300" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="cpu_nice" VAL="0.0" TYPE="float" UNITS="%" TN="13713221" TMAX="90" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="cpu_speed" VAL="2388" TYPE="uint32" UNITS="MHz" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="boottime" VAL="1235603852" TYPE="uint32" UNITS="s" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="cpu_wio" VAL="0.0" TYPE="float" UNITS="%" TN="13713221" TMAX="90" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="os_name" VAL="Linux" TYPE="string" UNITS="" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="load_one" VAL="0.28" TYPE="float" UNITS=" " TN="13713161" TMAX="70" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="os_release" VAL="2.6.25-14.fc9.x86_64" TYPE="string" UNITS="" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="disk_total" VAL="7.597" TYPE="double" UNITS="GB" TN="13713701" TMAX="1200" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="cpu_idle" VAL="99.8" TYPE="float" UNITS="%" TN="13713221" TMAX="90" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="cpu_user" VAL="0.1" TYPE="float" UNITS="%" TN="13713221" TMAX="90" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="swap_free" VAL="786232" TYPE="uint32" UNITS="KB" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="pkts_out" VAL="0.70" TYPE="float" UNITS="packets/sec" TN="13713401" TMAX="300" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="mem_cached" VAL="152740" TYPE="uint32" UNITS="KB" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="load_five" VAL="1.31" TYPE="float" UNITS=" " TN="13713161" TMAX="325" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="cpu_num" VAL="1" TYPE="uint16" UNITS="CPUs" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="load_fifteen" VAL="0.95" TYPE="float" UNITS=" " TN="13713161" TMAX="950" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="mem_free" VAL="95884" TYPE="uint32" UNITS="KB" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="cpu_system" VAL="0.1" TYPE="float" UNITS="%" TN="13713221" TMAX="90" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="proc_run" VAL="0" TYPE="uint32" UNITS=" " TN="13713221" TMAX="950" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="mem_total" VAL="381264" TYPE="uint32" UNITS="KB" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="cpu_aidle" VAL="11.4" TYPE="float" UNITS="%" TN="13713221" TMAX="3800" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="bytes_in" VAL="92.47" TYPE="float" UNITS="bytes/sec" TN="13713401" TMAX="300" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="mem_buffers" VAL="14336" TYPE="uint32" UNITS="KB" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="mem_shared" VAL="0" TYPE="uint32" UNITS="KB" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
<METRIC NAME="swap_total" VAL="786424" TYPE="uint32" UNITS="KB" TN="13713701" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond"/>
<METRIC NAME="part_max_used" VAL="59.2" TYPE="float" UNITS="%" TN="13713261" TMAX="180" DMAX="0" SLOPE="both" SOURCE="gmond"/>
</HOST>

</CLUSTER>
</GRID>
</GANGLIA_XML>
"""
  ganglia_31_xml = """<?xml version="1.0" encoding="ISO-8859-1" standalone="yes"?>
<!DOCTYPE GANGLIA_XML [
   <!ELEMENT GANGLIA_XML (GRID|CLUSTER|HOST)*>
      <!ATTLIST GANGLIA_XML VERSION CDATA #REQUIRED>
      <!ATTLIST GANGLIA_XML SOURCE CDATA #REQUIRED>
   <!ELEMENT GRID (CLUSTER | GRID | HOSTS | METRICS)*>
      <!ATTLIST GRID NAME CDATA #REQUIRED>
      <!ATTLIST GRID AUTHORITY CDATA #REQUIRED>
      <!ATTLIST GRID LOCALTIME CDATA #IMPLIED>
   <!ELEMENT CLUSTER (HOST | HOSTS | METRICS)*>
      <!ATTLIST CLUSTER NAME CDATA #REQUIRED>
      <!ATTLIST CLUSTER OWNER CDATA #IMPLIED>
      <!ATTLIST CLUSTER LATLONG CDATA #IMPLIED>
      <!ATTLIST CLUSTER URL CDATA #IMPLIED>
      <!ATTLIST CLUSTER LOCALTIME CDATA #REQUIRED>
   <!ELEMENT HOST (METRIC)*>
      <!ATTLIST HOST NAME CDATA #REQUIRED>
      <!ATTLIST HOST IP CDATA #REQUIRED>
      <!ATTLIST HOST LOCATION CDATA #IMPLIED>
      <!ATTLIST HOST REPORTED CDATA #REQUIRED>
      <!ATTLIST HOST TN CDATA #IMPLIED>
      <!ATTLIST HOST TMAX CDATA #IMPLIED>
      <!ATTLIST HOST DMAX CDATA #IMPLIED>
      <!ATTLIST HOST GMOND_STARTED CDATA #IMPLIED>
   <!ELEMENT METRIC (EXTRA_DATA*)>
      <!ATTLIST METRIC NAME CDATA #REQUIRED>
      <!ATTLIST METRIC VAL CDATA #REQUIRED>
      <!ATTLIST METRIC TYPE (string | int8 | uint8 | int16 | uint16 | int32 | uint32 | float | double | timestamp) #REQUIRED>
      <!ATTLIST METRIC UNITS CDATA #IMPLIED>
      <!ATTLIST METRIC TN CDATA #IMPLIED>
      <!ATTLIST METRIC TMAX CDATA #IMPLIED>
      <!ATTLIST METRIC DMAX CDATA #IMPLIED>
      <!ATTLIST METRIC SLOPE (zero | positive | negative | both | unspecified) #IMPLIED>
      <!ATTLIST METRIC SOURCE (gmond) 'gmond'>
   <!ELEMENT EXTRA_DATA (EXTRA_ELEMENT*)>
   <!ELEMENT EXTRA_ELEMENT EMPTY>
      <!ATTLIST EXTRA_ELEMENT NAME CDATA #REQUIRED>
      <!ATTLIST EXTRA_ELEMENT VAL CDATA #REQUIRED>
   <!ELEMENT HOSTS EMPTY>
      <!ATTLIST HOSTS UP CDATA #REQUIRED>
      <!ATTLIST HOSTS DOWN CDATA #REQUIRED>
      <!ATTLIST HOSTS SOURCE (gmond | gmetad) #REQUIRED>
   <!ELEMENT METRICS (EXTRA_DATA*)>
      <!ATTLIST METRICS NAME CDATA #REQUIRED>
      <!ATTLIST METRICS SUM CDATA #REQUIRED>
      <!ATTLIST METRICS NUM CDATA #REQUIRED>
      <!ATTLIST METRICS TYPE (string | int8 | uint8 | int16 | uint16 | int32 | uint32 | float | double | timestamp) #REQUIRED>
      <!ATTLIST METRICS UNITS CDATA #IMPLIED>
      <!ATTLIST METRICS SLOPE (zero | positive | negative | both | unspecified) #IMPLIED>
      <!ATTLIST METRICS SOURCE (gmond) 'gmond'>
]>

<GANGLIA_XML VERSION="3.1.2" SOURCE="gmond">
−
<CLUSTER NAME="unspecified" LOCALTIME="1249085211" OWNER="unspecified" LATLONG="unspecified" URL="unspecified">
−
<HOST NAME="192.168.88.156" IP="192.168.88.156" REPORTED="1248923974" TN="161237" TMAX="20" DMAX="0" LOCATION="unspecified" GMOND_STARTED="1248886163">
−
<METRIC NAME="load_one" VAL="0.15" TYPE="float" UNITS=" " TN="161232" TMAX="70" DMAX="0" SLOPE="both" SOURCE="gmond">
−
<EXTRA_DATA>
<EXTRA_ELEMENT NAME="GROUP" VAL="load"/>
<EXTRA_ELEMENT NAME="DESC" VAL="One minute load average"/>
<EXTRA_ELEMENT NAME="TITLE" VAL="One Minute Load Average"/>
</EXTRA_DATA>
</METRIC>
−
<METRIC NAME="mem_total" VAL="4194304" TYPE="float" UNITS="KB" TN="169043" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond">
−
<EXTRA_DATA>
<EXTRA_ELEMENT NAME="GROUP" VAL="memory"/>
<EXTRA_ELEMENT NAME="DESC" VAL="Total amount of memory displayed in KBs"/>
<EXTRA_ELEMENT NAME="TITLE" VAL="Memory Total"/>
</EXTRA_DATA>
</METRIC>
−
<METRIC NAME="os_release" VAL="9.7.0" TYPE="string" UNITS="" TN="169043" TMAX="1200" DMAX="0" SLOPE="zero" SOURCE="gmond">
−
<EXTRA_DATA>
<EXTRA_ELEMENT NAME="GROUP" VAL="system"/>
<EXTRA_ELEMENT NAME="DESC" VAL="Operating system release date"/>
<EXTRA_ELEMENT NAME="TITLE" VAL="Operating System Release"/>
</EXTRA_DATA>
</METRIC>
−
<METRIC NAME="proc_run" VAL="3" TYPE="uint32" UNITS=" " TN="161283" TMAX="950" DMAX="0" SLOPE="both" SOURCE="gmond">
−
<EXTRA_DATA>
<EXTRA_ELEMENT NAME="GROUP" VAL="process"/>
<EXTRA_ELEMENT NAME="DESC" VAL="Total number of running processes"/>
<EXTRA_ELEMENT NAME="TITLE" VAL="Total Running Processes"/>
</EXTRA_DATA>
</METRIC>
−
<METRIC NAME="load_five" VAL="0.21" TYPE="float" UNITS=" " TN="161232" TMAX="325" DMAX="0" SLOPE="both" SOURCE="gmond">
−
<EXTRA_DATA>
<EXTRA_ELEMENT NAME="GROUP" VAL="load"/>
<EXTRA_ELEMENT NAME="DESC" VAL="Five minute load average"/>
<EXTRA_ELEMENT NAME="TITLE" VAL="Five Minute Load Average"/>
</EXTRA_DATA>
</METRIC>
−
<METRIC NAME="gexec" VAL="OFF" TYPE="string" UNITS="" TN="161243" TMAX="300" DMAX="0" SLOPE="zero" SOURCE="gmond">
−
<EXTRA_DATA>
<EXTRA_ELEMENT NAME="GROUP" VAL="core"/>
<EXTRA_ELEMENT NAME="DESC" VAL="gexec available"/>
<EXTRA_ELEMENT NAME="TITLE" VAL="Gexec Status"/>
</EXTRA_DATA>
</METRIC>
−
</HOST>
</CLUSTER>
</GANGLIA_XML>
"""

if __name__ == '__main__':
  unittest.main()
