#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Client interface towards healthd

Connects over HTTP to healthd and materializes the metrics as DataPoints
"""

import urllib2
import cPickle

from health.models import Participant
from health.data import DataPoint

# Global singleton instance
HEALTHD_CLIENT = None

class HealthdClient(object):
  def __init__(self, host, port):
    self.host = host
    self.port = port

  def _get_metrics_data(self):
    # TODO(todd) need a timeout here!
    try:
      u = urllib2.urlopen('http://%s:%d/?format=pickle' % (self.host, self.port))
    except:
      # TODO(todd) - it would be nice if the datapoint sources actually could generate
      # lints as well as datapoints, so it could generate a lint saying "healthd not responding"
      # in this case
      return
    data = u.read()
    return cPickle.loads(data)

  def get_fresh_datapoints(self):
    metrics = self._get_metrics_data()
    if not metrics:
      return

    # Load the entirety of participants into a dictionary rather than doing
    # a bunch of single-key gets, since we assume we're going to need them
    # all anyway.
    participants = dict((p.id, p) for p in Participant.objects.all())

    for p_id, host_data in metrics.iteritems():
      participant = participants.get(p_id)

      # It's possible that an old participant still has metrics, temporarily,
      # if the healthd is out of sync
      if not participant: continue
      for context, context_data in host_data.iteritems():
        for record_name, record_data in context_data.iteritems():
          dp = DataPoint(cluster=participant.cluster, participant=participant,
                         key="metrics:%s.%s" % (context, record_name),
                         value=record_data)
          yield dp

class MockHealthdClient(HealthdClient):
  def __init__(self, mock_data):
    self._mock_data = mock_data
    super(MockHealthdClient, self).__init__("fake-host", -1)

  def _get_metrics_data(self):
    return self._mock_data

def get_fresh_datapoints():
  global HEALTHD_CLIENT
  if not HEALTHD_CLIENT:
    HEALTHD_CLIENT = HealthdClient("127.0.0.1", 6473) # TODO(todd) conf this
  return HEALTHD_CLIENT.get_fresh_datapoints()
