#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

import unittest
from health import lint

class TestLint(unittest.TestCase):
  class TestCheck(lint.ThresholdLintCheck):
    id = "test_check"
    name = "Test Check"
    good_message = "Value was above five"
    bad_message = "Value was below five"
    warn_level = 10
    critical_level = 5
    comparison = '>='
    format_value = lint.ThresholdLintCheck.format_value_percent

  class TestRoller(lint.LintRoller):
    skip_registration = True
    def run_checks(self, datapoints):
      for d in datapoints:
        yield TestLint.TestCheck.check("test_subject", d)

  def test_basics(self):
    results = lint.run_lint_rollers([3,5,8,27], [TestLint.TestRoller])
    self.assertEquals(len(results), 4)

    self.assertEquals(results[0].id, 'test_check')
    self.assertEquals(results[0].status, lint.CRITICAL)
    self.assertEquals(results[0].message, TestLint.TestCheck.bad_message)

    self.assertEquals(results[1].id, 'test_check')
    self.assertEquals(results[1].status, lint.WARN)
    self.assertEquals(results[1].message, TestLint.TestCheck.bad_message)

    self.assertEquals(results[2].id, 'test_check')
    self.assertEquals(results[2].status, lint.WARN)
    self.assertEquals(results[2].message, TestLint.TestCheck.bad_message)

    self.assertEquals(results[3].id, 'test_check')
    self.assertEquals(results[3].status, lint.OK)
    self.assertEquals(results[3].message, TestLint.TestCheck.good_message)

  def test_registration(self):
    from health import lint_checks
    self.assertTrue(len(lint.ROLLER_REGISTRY) > 0)

if __name__ == '__main__':
  unittest.main()
