#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
from health import report
from health.models import DATA_NODE, HDFS, Participant
from health.clusters import Cluster
from health.data import DataPoint
from health import extractors
from nose.tools import *

def test_report():
  dps = []
  
  hdfs = Cluster(type=HDFS, name="myhdfs")

  # Set up an HDFS cluster with 10 datanodes, most of them at 2000ms
  # and one of them 30sec
  DEFAULT_VAL=2000
  values = {1: 1000,
            3: 90000}
  states = {5: 'DOWN'} # make one node down
  dn_participants = []
  for i in xrange(0, 10):
    dn = Participant(id=i,
                     cluster_type=HDFS, cluster_name="myhdfs",
                     role=DATA_NODE, ip="1.2.3.%d" % i, name="dn-%d" % i)
    dn_participants.append(dn)

    dps.append(DataPoint(cluster=hdfs, participant=dn, key="state",
                         value=states.get(i, 'UP')))
    dps.append(DataPoint(cluster=hdfs, participant=dn,
                         key="datanode_report:millisSinceUpdate",
                         value=values.get(i, DEFAULT_VAL)))

  rep = report.Report(datapoints=dps,
                      extractors=[extractors.dn_last_seen,
                                  extractors.node_up])

  # This should show high intensity for the 30sec one
  bad_part_cells = rep.cells_by_participant[dn_participants[3]]
  cell = bad_part_cells[extractors.dn_last_seen]
  assert_equals(rep.get_intensity_for_cell(cell), 1.0)

  # And low intensity for one of the others
  good_part_cells = rep.cells_by_participant[dn_participants[1]]
  cell = good_part_cells[extractors.dn_last_seen]
  assert_equals(cell.value, 1000)
  assert_equals(rep.get_intensity_for_cell(cell), 0.0)

  # Check to make sure down node is red
  cell = rep.cells_by_participant[dn_participants[5]][extractors.node_up]
  assert_equals(cell.value, 0.0) # node down
  assert_equals(rep.get_intensity_for_cell(cell), 1.0) # red

  # And up node is green
  cell = rep.cells_by_participant[dn_participants[8]][extractors.node_up]
  assert_equals(cell.value, 1.0) # node up
  assert_equals(rep.get_intensity_for_cell(cell), 0.0) # green
