#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Tests for the health views.

Very minimal so far.
"""
from django.test.client import Client
from nose.tools import assert_true, assert_equals
from nose.plugins.attrib import attr
import simplejson

from hadoop import mini_cluster

@attr('requires_hadoop')
def test_hadoop_views():
  """
  Tests that views that require talking to Hadoop are 
  reasonable.
  """
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = Client()
    # All apps require login.
    c.login(username="test", password="test")

    # Test basic output.
    response_raw = c.get('/health/', dict(format="json"))
    response = simplejson.JSONDecoder().decode(response_raw.content)
    # 2 are HDFS and MR; Ganglia is sometimes around.
    assert_true(len(response["clusters"]) >= 2)
    matches = 0
    for x in response["clusters"]:
      if x["type"] == "HDFS":
        assert_equals(2, x["state_counts"]["UP"])
        matches += 1
    assert_equals(1, matches)

    # Basic success for MR page
    response = c.get('/health/cluster/MR/default')
    assert_equals(200, response.status_code)

    # Basic success for HDFS page
    response = c.get('/health/cluster/HDFS/default')
    assert_equals(200, response.status_code)
  finally:
    cluster.shutdown()

def test_no_hadoop():
  """
  Tests that things work when Hadoop is down
  """
  c = Client()
  # All apps require login.
  c.login(username="test", password="test")

  # Test basic output.
  response_raw = c.get('/health/', dict(format="json"))
  response = simplejson.JSONDecoder().decode(response_raw.content)
  # 2 are HDFS and MR; Ganglia is sometimes around.
  assert_true(len(response["clusters"]) >= 2)

  # Test a host when it's down
  response_raw = c.get('/health/host/127.0.0.1')
  assert_equals(200, response_raw.status_code)


@attr('requires_hadoop')
def test_lint_views():
  """
  Tests lint views against a real Hadoop
  """
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = Client()
    # All apps require login.
    c.login(username="test", password="test")

    # Test basic output.
    response_raw = c.get('/health/lints', dict(format="json"))
    response = simplejson.JSONDecoder().decode(response_raw.content)
    assert_true(len(response["lints"]) >= 4)
  finally:
    cluster.shutdown()

@attr('requires_hadoop')
def test_heatmap_view():
  """
  Tests lint views against a real Hadoop
  """
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = Client()
    # All apps require login.
    c.login(username="test", password="test")

    # Test that the view at least renders
    response_raw = c.get('/health/heatmap')
    assert_equals(200, response_raw.status_code)
  finally:
    cluster.shutdown()

@attr('requires_hadoop')
def test_perhost_view():
  """
  Tests the per-host view for the minicluster
  """
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = Client()
    # All apps require login.
    c.login(username="test", password="test")

    # Test that the view at least renders
    response_raw = c.get('/health/host/127.0.0.1')
    assert_equals(200, response_raw.status_code)
  finally:
    cluster.shutdown()
