// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Cluster health
provides: [CCS.HealthDashboard]
requires: [ccs-shared/CCS.JBrowser, clientcide/MooScroller, More/Element.Delegation,
  clientcide/Collapsible]
script: CCS.HealthDashboard.js

...
*/

(function(){

ART.Sheet.defineStyle('window.health_dashboard', {
		'header-height': 60,
		'min-width': 350,
		'min-height': 260
});
ART.Sheet.defineStyle('window.health_dashboard:dragging', {
	'content-visibility': 'visible'
});

CCS.HealthDashboard = new Class({

	Extends: CCS.JBrowser.Solid, 

	options: {
		'className': 'smoke solid health_dashboard',
		minimize: false,
		maximize: false,
		width: 350,
		height: 300,
		position: 'upperRight',
		edge: 'upperRight',
		relativeTo: 'ccs-desktop',
		offset: {
			x: -10,
			y: 10
		},
		jframeOptions: {
			useSpinner: false,
			onLoadError: function(){
				CCS.error('DashBoard', 'The Dashboard application could not reach the server. It has stopped auto-refreshing; you might want to restart it.');
			}
		},
		windowTitler: function(){
			return '';
		}
	},

	initialize: function(path, options) {
		this.parent(path || "/health/dashboard", options);
		$(this.jframe).setStyle('overflow', 'hidden');
		this.healthFilters = new HealthFilters();
		this.addEvent('load', this.setup.bind(this));
	},

	setup: function(view){
		this.setCaption('');
		this.removeEvent('shade', this.boundResize);
		if (view == 'health_dashboard') this.setupDashboard();
		if (view == 'dashboard_lints') this.setupLints();
	},

	setupDashboard: function(){
		if (this.scroller) this.scroller.detach();
		this.map = $(this).getElement('.health_heatmap').setStyle('visibility', 'hidden');
		var scroller = $(this).getElement('.health_heatmap_scroller div');
		this.scroller = new MooScroller(this.map, scroller);
		var tip = $(this).getElement('.health_heatmap_tips .health_heatmap_tip');
		var machine = $(this).getElement('.health_heatmap_tips .health_heatmap_machine');
		var empty = function(){
			tip.empty();
			machine.empty();
		};
		this.map.addEvents({
			'mouseover:relay(td a.heat-spot)': function(e, el){
				if (!el) return;
				var title, ip;
				if (el.retrieve('heatmap:ip')) {
					ip = el.retrieve('heatmap:ip');
					title = el.retrieve('heatmap:tip');
				} else {
					var text = el.get('title');
					if (text && text.contains(' | ')) {
						ip = text.split(' | ')[0];
						title = text.split(' | ')[1];
					} else {
						ip = text;
					}
					el.store('heatmap:tip', title).store('heatmap:ip', ip).set('title', '');
					el.addEvent('mouseleave', empty);
				}
				if (title) tip.set('html', 'status: ' + title);
				machine.set('html', 'host: ' + ip);
			}
		});
		this.healthFilters.attach(this.map, $(this).getElement('ul.health_filters'));
		this.map.setStyle('visibility', 'visible');
		this.resizeMap();
		this.boundResize = this.resizeMap.bind(this);
		this.addEvent('shade', this.boundResize);
	},
	
	setupLints: function(){
		this.lints = $(this).getElement('.health_lints');
		var scroller = $(this).getElement('.health_lints_scroller div');
		this.scroller = new MooScroller(this.lints, scroller);
		this.boundResize = this.resizeLints.bind(this);
		this.addEvent('resize:move', this.boundResize);
		this.resizeLints();
	},

	makeButtons: function(){
		this.parent.apply(this, arguments);
		$(this.buttons.close).setStyles({
			'right': 6,
			'left': 'auto'
		});
	},

	resizeMap: function(dragging){
		if (!dragging) {
			this.map.setStyle('height', this.contentSize.y - 155);
			this.scroller.update();
		}
	},

	resizeLints: function() {
		this.lints.setStyle('height', this.contentSize.y - 40);
		this.scroller.update();
	}

});


var HealthFilters = new Class({
	attach: function(table, filters) {
		this.table = $(table);
		this.filters = $(filters);
		this.filters.addEvent('click:relay(a)', this.toggleFilter.bind(this));
		this.filters.getElements('a').each(function(a) {
			var text = a.get('html').trim();
			var checked = $pick(this.state[text], true);
			if (!checked) {
				a.removeClass('health_checked');
				this.filter(text, checked);
			}
		}, this);
	},
	toggleFilter: function(e, link){
		var checked = link.toggleClass('health_checked').hasClass('health_checked');
		this.filter(link.get('html'), checked);
	},
	
	state: {},
	
	filter: function(text, include) {
		this.state[text] = include;
		switch(text.trim()) {
			case 'healthy':
			case 'troubled':
			case 'critical':
				this.filterByState(text, include);
				break;
			default:
				this.filterByRole(text, include);
		}
	},
	
	filterByState: function(state, include) {
		var rows = $$([]);
		if (state == "healthy") {
			this.table.getElements('tr').each(function(tr){
				var healthy = tr.getElements('a.heat-spot').every(function(a) {
					return !a.hasClass('troubled') && !a.hasClass('critical');
				});
				if (healthy) rows.push(tr);
			});
		} else {
			this.table.getElements('a.' + state).each(function(td) {
				rows.include(td.getParent('tr'));
			});
		}
		if (!include) rows.setStyle('display', 'none');
		else rows.setStyle('display', 'table-row');
	},
	
	filterByRole: function(role, include) {
		this.table.getElements('tr').each(function(tr){
			var abv = tr.getElement('span.health_role_abv');
			if (abv.get('html').trim().toLowerCase() == role.toLowerCase()) tr.setStyle('display', include ? 'table-row' : 'none');
		});
	}
	
});

})();
