// (c) Copyright 2010 Cloudera, Inc.
package com.cloudera.jobsub;

import org.apache.hadoop.security.UnixUserGroupInformation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Overrides the 'whoami' and 'groups' calls that Hadoop does with
 * environment variables, specifying the username and group.
 */
public aspect UgiFixer {
  private static final Log LOG = LogFactory.getLog(UgiFixer.class);
  private static final String USER_VAR = "CLOUDERA_JOBSUB_USER";
  private static final String DEFAULT_USER = "default_jobsub_user";
  
  // TODO(philip): Full-on group support would mean
  // dealing with comma-delimited values here.
  private static final String GROUPS_VAR = "CLOUDERA_JOBSUB_GROUPS";
  private static final String DEFAULT_GROUP = "default_jobsub_group";

  UgiFixer() {
    LOG.info("Cloudera UGI fixer aspect loaded.");
  }

  pointcut getUserName():
        call(String UnixUserGroupInformation.getUnixUserName());

  pointcut getUnixGroups():
        call(String[] UnixUserGroupInformation.getUnixGroups());

  String around(): getUserName() {
    String user = System.getenv(USER_VAR);
    return user == null ? DEFAULT_USER : user;
  }

  String[] around(): getUnixGroups() {
    String group = System.getenv(GROUPS_VAR);
    if (group == null) {
      return new String[] { DEFAULT_GROUP };
    } else {
      return group.split(",");
    }
  }
}
