/*
---
description: Visualization Tools
provides: [FS.Visualize]
requires: [ccs-shared/CCS.JFrame, ccs-shared/CCS.Request, art-widgets/ART.Window,
  ccs-shared/EditInline, More/URI, clientcide/StickyWin.Alert, clientcide/StickyWin.Prompt,
  More/Date, art-widgets/ART.SplitView, protovis/MooVis, More/Fx.Accordion, More/HtmlTable]
script: FS.Visualize.js

...
*/
ART.Sheet.defineStyle('window.visualizer', {
	'header-height': 60,
	'header-overflow': 'visible',
	'min-width': 500
});

FS.Visualize = new Class({

	Extends : CCS.JBrowser,
	
	options : {
		'baseUrl' : '/visualizer/',
		'fileUrl': '/filebrowser/view/',
		'className' : 'visualizer',
		'filesystem' : 'default',
		'displayHistory' : false
	},

	graph : null, // graph not defined yet
	graphType : null, // checks to see if we need to render a new graph type	
	
	initialize : function(data, options){
		this.path = data.path;
		this.size = data.stats.size;
		this.parent(this.options.baseUrl, options);
		this.filesystem = FS.getFileSystem(this.options.filesystem);
		this.setCaption(this.path+' -- Visualizer'); // set Window Title
		
		// Set buttons in header	
		new Element('div',{
			'class' : 'fb_viewer_header',
			'html' : '<ul class="fb_buttons">'+
						'<div class="selected_action_buttons">'+
						'<li class="fb_data"><a>rename</a></li><li class="fb_visualize"><a>graph</a></li>'+
					 '</div></ul>'
			}).inject(this.header);

		this.addEvent('resize:end', function(){ 
			if(this.graph) this.graph.refresh(); 
		}.bind(this));
		
	},
	
	_jframeLoaded: function(data) {
		this.setup.delay(10,this); // this had to be added or else the DOM wasn't free yet from jFrame
		this.fireEvent('load', data.view);
	},
	
	setup : function(){
		// TODO: use jFrame, resolve bugs involved
		this.split = new ART.SplitView();
		this.addEvent('resize', function(w, h) {;
			this.split.resize(this.contentSize.x, this.contentSize.y);
		}.bind(this));
		this.split.setLeftContent($(this).getElement('.left_col'));
		this.split.setRightContent($(this).getElement('.right_col'));
		this.split.setParent(this);
		this.split.resizeLeft(265);
		this.content.getElement('.visualizer_content').adopt($(this.split));
		this.split.resize(this.contentSize.x, this.contentSize.y);		
		
		// port to jframe? 
		new Fx.Accordion(
			this.content.getElements('.vis_wiz h2'),
			this.content.getElements('.vis_wiz div.content'),{
				'height' : true
		});	
		
		// DOM area references
		this.contentArea = this.element.getElement('.right_col').addClass('vis_app');
		this.dataArea = new Element('div').inject(this.contentArea);
		this.graphArea = new Element('div').inject(this.contentArea);	
		this.wizArea = this.element.getElement('div.vis_wiz');
		
		this.configureWizard();
		this.load();
				
	},
	
	load: function(callback) {
		new Request.JSON({
			useSpinner: true,
			spinnerTarget: this.container,
			spinnerOptions: { fxOptions: {duration: 200} },
			url: new URI(FS.pathJoin(this.options.fileUrl, this.path)).setData({
				format: 'json',
				length: this.size
			}).toString(),
			onCcsError: function(error){
				// TODO(todd) should we really call callback on error?
				if(callback) callback.apply(this, this.path);
			}.bind(this),
			onSuccess: function(data) {
				this.file = new FS.Files.File(data, this.filesystem);
				this.renderContents();
				this.fireEvent('dataLoaded', [data, parent]);
				if(callback) callback.apply(this, this.path);
			}.bind(this)
		}).send();
	},
	
	renderContents: function(){
		this.data = this.parseCSV(this.file.data.view.contents);
		this.renderTable();
	},
	
	renderTable : function(){
		
		this.table = new HtmlTable({
			properties: { border: 1 },
			rows : this.data
		}).inject(this.dataArea);

		this.tableRendered = true;		
	},

	configureWizard : function(){
		
		var el = this.element;
		this.dataBtn = el.getElement('.fb_buttons li.fb_data');
		this.graphBtn = el.getElement('.fb_buttons li.fb_visualize');
		
		this.graphTypes = el.getElements('div.vis_wiz input[type=radio]');
		
		this.width = el.getElement('div.vis_wiz input[name=width]');
		this.height = el.getElement('div.vis_wiz input[name=height]');
		
		this.top = el.getElement('div.vis_wiz input[name=top]');
		this.bottom = el.getElement('div.vis_wiz input[name=bottom]');
		this.left = el.getElement('div.vis_wiz input[name=left]');
		this.right = el.getElement('div.vis_wiz input[name=right]');
		
		
		this.graphBtn.addEvent('click',this.showGraph.bind(this));
		this.dataBtn.addEvent('click',this.showData.bind(this));
	},
	
	getOptions : function(){
				
		var opts = {
			data : this.data,
			top : this.top.value.toInt(),
			bottom : this.bottom.value.toInt(),
			left  : this.left.value.toInt(),
			right : this.right.value.toInt()
		};
		
		if(this.width.value == 'auto' || this.height.value == 'auto') $extend(opts, {width : 'auto', height : 'auto'});
		else $extend(opts, {width : this.width.value.toInt(), height : this.height.value.toInt() });
		
		// auto binding of input fields for graph options
		this.wizArea.getElements('.other-'+this.graphType+' input').each(function(other){
			switch(other.type){
				case 'checkbox' : 
					opts[other.name] = (other.checked) ? true : false;
					break;
				default : 
					opts[other.name] = other.value.toInt();
					break;
			}
		},this);
		
		this.wizArea.getElements('.axis-'+this.graphType+' input').each(function(other){
			switch(other.type){
				case 'checkbox' : 
					opts[other.name] = (other.checked) ? true : false;
					break;
				default : 
					opts[other.name] = other.value;
					break;
			}
		},this);
	
		return opts;
		
	},
	
	showGraph : function(){

		this.graphType = null;
		this.graphTypes.each(function(gtype){
			if(gtype.checked) this.graphType = gtype.value;
		},this);
		
		if(this.graphType == null) return StickyWin.alert('Graph', 'Error: no graph type selected');
		
		this.graphArea.empty();
		this.graph = new Protovis[this.graphType](this.graphArea, this.getOptions());	
		
		this.dataArea.setStyle('display','none');
		this.graphArea.setStyle('display','block');
	},
	
	showData : function(){
		this.graphArea.setStyle('display','none');
		this.dataArea.setStyle('display','block');
	},	

	//masks the file list area
	mask: function(){
		this.element.spin();
		return this;
	},
	
	unmask: function(){
		this.element.unspin();
		return this;
	},	
	
	resize: function(w, h) {
		this.parent(w, h);
		this.jframe.resize(this.contentSize.x, this.contentSize.y);
	},

	// still needs to be reformated. This is being removed so no more time is being spent.
	parseCSV: function(csvString) {
        var fieldEndMarker = /([,\015\012] *)/g; /* Comma is assumed as field separator */
        var qFieldEndMarker = /("")*"([,\015\012] *)/g; /* Double quotes are assumed as the quote character */
        var startIndex = 0;
        var records = [], currentRecord = [];
        do {
            // If the to-be-matched substring starts with a double-quote, use the qFieldMarker regex, otherwise use fieldMarker.
            var endMarkerRE = (csvString.charAt(startIndex) == '"') ? qFieldEndMarker : fieldEndMarker;
            endMarkerRE.lastIndex = startIndex;
            var matchArray = endMarkerRE.exec(csvString);
            if (!matchArray || !matchArray.length) {
                break;
            }
            var endIndex = endMarkerRE.lastIndex - matchArray[matchArray.length - 1].length;
            var match = csvString.substring(startIndex, endIndex);
            if (match.charAt(0) == '"') { // The matching field starts with a quoting character, so remove the quotes
                match = match.substring(1, match.length - 1).replace(/""/g, '"');
            }
            currentRecord.push(match);
            var marker = matchArray[0];
            if (marker.indexOf(',') < 0) { // Field ends with newline, not comma
                records.push(currentRecord);
                currentRecord = [];
            }
            startIndex = endMarkerRE.lastIndex;
        }
        while (true);
        if (startIndex < csvString.length) { // Maybe something left over?
            var remaining = csvString.substring(startIndex).trim();
            if (remaining) 
                currentRecord.push(remaining);
        }
        if (currentRecord.length > 0) { // Account for the last record
            records.push(currentRecord);
        }
        return records;
	}	

});
