// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Desktop Configuration
provides: [CCS.Desktop.Config]
requires: [/CCS.Desktop, clientcide/StickyWin.Drag]
script: CCS.Desktop.Config.js

...
*/
//by default, make all StickyWin instances load inside of div#ccs-desktop
StickyWin.implement({
	options: {
		inject: {
			target: 'ccs-desktop'
		}
	},
	destroyOnClose: true
});

//same for ART.StickyWin instances
ART.StickyWin.implement({
	destroyOnClose: true,
	options: {
		cascaded: true,
		inject: {
			target: 'ccs-desktop'
		}
	}
});
ART.Sheet.defineStyle('window', {
	'min-height': 88
});
window.addEvent('domready', function(){
	//this sizer method will ensure that windows are always reachable and ever larger than the window
	//(so you can always reach it's bottom corner to resize it)
	var sizer = function(){
		//get the size of the desktop itself
		var size = $('ccs-desktop').getSize();
		//minus the offset of the desktop
		size.y = size.y - $('ccs-desktop').getStyle('top').toInt();
		//define the default size of windows to be 70% of the window size
		var styles = ART.Sheet.lookupStyle('window');
		ART.Sheet.defineStyle('window', {
			'height': (size.y * 0.7).toInt(),
			'width': (size.x * 0.7).toInt(),
			//and the max height/width is the size of the desktop
			//minus 35 px for the height for the dock
			'max-height': Math.max(styles.minHeight, (size.y + 2).toInt() - 35),
			'max-width': Math.max(styles.minWidth, (size.x + 2).toInt())
		});
		//for all the existing instances, resize them and move them onto the screen.
		ART.StickyWin.DefaultManager.instances.each(function(instance) {
			var resize = {
				height: Math.max(styles.minHeight, Math.min(instance.currentHeight, size.y + 2)),
				width: Math.max(styles.minWidth, Math.min(instance.currentWidth, size.x + 2))
			};
			instance.resize(resize.width, resize.height);
			instance.getOnScreen();
		});
	};
	var timer;
	window.addEvent('resize', function(){
		$clear(timer);
		//when we resize the window, we adjust the open windows, but this event - window's resize event -
		//fires constantly as you resize it, so we add a timeout that is reset every time the event fires
		timer = sizer.delay(100);
	});
	//size everything now
	sizer();
	//whenever the window scrolls down (if the user hits space, for example, or page down)
	//send them back to 0,0; the desktop isn't supposed to scroll
	window.addEvent('scroll', function(e) {
		window.scrollTo(0,0);
	});
});
