// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Keyboard shortcuts for Desktop
provides: [CCS.Desktop.Keys]
requires: [More/Keyboard.Extras, art-widgets/ART.StickyWin, More/HtmlTable, More/Element.Shortcuts]
script: CCS.Desktop.Keys.js

...
*/
/*
	adds window level keys to desktop as well as a method to display the active keys in use.
*/
(function(){

	var keyShower;

	//prevent accidental scrolling
	var stopper = function(e) {
		if (e.target.tagName.toLowerCase() == 'html') e.stop();
	};
	Keyboard.manager.addEvents({
		'space': stopper,
		'down': stopper,
		'pagedown': stopper,
		'end': stopper,
		'keydown:backspace': stopper
	});


	Keyboard.manager.addShortcuts({
		'Previous Window': {
			keys: 'ctrl+left',
			shortcut: 'control (or alt) + left',
			handler: function(e){
				ART.StickyWin.DefaultManager.cycle('back', 'default');
				Keyboard.stop(e);
			},
			description: 'Bring the previous window to the foreground.'
		},
		'Next Window': {
			keys: 'ctrl+right',
			shortcut: 'control (or alt) + right',
			handler: function(e){
				ART.StickyWin.DefaultManager.cycle('forward', 'default');
				Keyboard.stop(e);
			},
			description: 'Bring the next window to the foreground.'
		},
		'New Window': {
			keys: 'ctrl+shift+n',
			shortcut: 'control (or alt) + shift + n',
			handler: function(e){
				var app = $(ART.StickyWin.DefaultManager.enabled).retrieve('CCS:component');
				if (app) CCS.Desktop.launch(app);
				Keyboard.stop(e);
			},
			description: 'Launch a new window for the current application (if it allows it).'
		},
		'Close Window': {
			keys: 'ctrl+shift+w',
			shortcut: 'control (or alt) + shift + w',
			handler: function(e){
				if (ART.StickyWin.DefaultManager.enabled) ART.StickyWin.DefaultManager.enabled.hide();
				Keyboard.stop(e);
			},
			description: 'Close the current window.'
		},
		'Show Shortcuts': {
			keys: 'ctrl+shift+/',
			shortcut: 'control + "?" (shift + "/")',
			handler: function(e){
				keyShower.toggle();
				Keyboard.stop(e);
			},
			description: 'Show a list of all active shortcuts.'
		}
	});
	Keyboard.manager.addEvents({
		'alt+left': function(){
			ART.StickyWin.DefaultManager.cycle('back', 'default');
			Keyboard.stop(e);
		},
		'alt+right': function(){
			ART.StickyWin.DefaultManager.cycle('forward', 'default');
			Keyboard.stop(e);
		},
		'alt+shift+n': function(e){
			var app = $(ART.StickyWin.DefaultManager.enabled).retrieve('CCS:component');
			if (app) CCS.Desktop.launch(app);
			Keyboard.stop(e);
		},
		'alt+shift+w': function(e){
			if (ART.StickyWin.DefaultManager.enabled) ART.StickyWin.DefaultManager.enabled.hide();
			Keyboard.stop(e);
		}
	});

	//functionality to display the shortcuts to the user.
	ART.StickyWin.DefaultManager.setLayer('shortcuts', 150);

	CCS.KeyList = new Class({

		Implements: Events,

		initialize: function(){
			//create a table for the active shortcuts
			this.table = new HtmlTable({
				headers: ['Key', 'Shortcut Name', 'Description'],
				properties: {
					'class':'ccs-data_table'
				}
			});
			//a holder for styling
			this.win = new Element('div', {
				'id':'ccs-shortcut_list',
				styles: {
					display: 'none'
				}
			}).inject(document.body).adopt(this.table);
			//a local instance of Mask for the document
			this.mask = new Mask(document.body, {
				hideOnClick: true,
				onHide: this.hide.bind(this, true)
			});
		},

		//show the shortcut list
		show: function(){
			if (this.win.isDisplayed()) return;
			this.fill();
			this.mask.show();
			this.win.show().position();
			this.fireEvent('show');
		},

		//hide the shortcut list
		hide: function(dontHideMask){
			if (!this.win.isDisplayed()) return;
			if (dontHideMask) {
				this.win.hide();
				this.fireEvent('hide');
			} else {
				this.mask.hide();
			}
		},

		//toggle visibility of the shortcut list
		toggle: function(){
			if (this.win.isDisplayed()) this.hide();
			else this.show();
		},

		//fills the shortcut element with a list of active shortcuts
		fill: function(){
			var keys = Keyboard.getActiveShortcuts();
			this.table.empty();
			var wname;
			keys.reverse().each(function(key){
				var widget = key.getKeyboard().widget;
				var widgetName = widget ? CCS.Desktop.getAppName(CCS.Desktop.getAppComponentName(widget)) : "Desktop";
				if (wname != widgetName) {
					wname = widgetName;
					this.table.push([{
						content: wname,
						properties: {
							colspan: 3,
							styles: {
								'font-weight': 'bold'
							}
						}
					}]);
				}
				this.table.push([key.shortcut, key.name, key.description || ""]);
			}, this);
		}

	});

	window.addEvent('domready', function(){
		keyShower = new CCS.KeyList();
		$('hotkeys').addEvent('click', function(){
			keyShower.show();
		});
	});

})();
