// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Core CCS.Desktop Functionality.
provides: [CCS.Desktop]
requires: [ccs-shared/CCS, depender_client/Depender.Client, core/Request, /CCS.Dock,
  More/Elements.From, art-widgets/ART.Window, More/URI]
script: CCS.Desktop.js

...
*/
CCS.Desktop = {
	
	options: {
	//	onBeforeLoad: $empty(componentName)
	//	onComponentLoaded: $empty(componentName)
	//	onBeforeLaunch: $empty(componentName)
	//	onAfterLaunch: $empty(componentName)
	//	loginUrl: 'login.html', TODO(nutron)
		append: 'ccs-desktop'
	},
	
	initialize: function(options){
		this.setOptions(options);
		this.addEvent('componentLoaded', function(component){
			this.loadedComponents[component] = true;
		}.bind(this));
		CCS.Dock.initialize();
	},

	loaders: {},
	launchers: {},
	loadedComponents: {},
	stateUrl: "prefs/state",

	/******************
	 * PUBLIC METHODS
	 ******************/

	//registers an object of key/value components
	/* example: 
	CCS.Desktop.register({
		DocViewer: {											//app name
			autolaunch: true/false, //if true, the app launches at registration time (startup)
			css: '/path/to/css/file.css',
			html: '/docviewer/templates/doc_viewer_body.html',	//template asset
			require: ['DV.Viewer'],								//required js files
			launch: function(path){								//function to execute whenever app is launched
				var dv = CCS.Desktop.injectTemplate('DocViewer');
				return new DV.Viewer(dv, path);
			},
			menu: {												//menu item (optional)
				id: 'ccs-filebrowser-menu',						//id of the item
				img: {											//img properties (optional)
					src: '/jobtracker/static/art/jobtracker-small.png',
					width: 34,
					height: 33
				},
				name: 'Doc Viewer'								//text to display
			}
		}
	}); */
	register: function(components) {
		$each(components, function(val, key){
			this.add(key, val);
		}, this);
		return this;
	},

	bootstraps: {},

	getBootstrap: function(component){
		return this.bootstraps[component];
	},

	//adds a single component (string - the app name) and its options (object)
	//see register example above
	add: function(component, options) {
		if (!options || this.loaders[component]) return this;
		this.bootstraps[component] = options;
		//if there is a custom launch method, set the launcher
		if (options.launch) this.setLauncher(component, options.launch);
		this.loaders[component] = function(callback){
			//if there is a css link, inject it
			if (options.css && !$$('link').get('href').contains(options.css)) {
				new Element('link', {
					rel: 'stylesheet', 
					media: 'screen', 
					type: 'text/css', 
					href: options.css
				}).inject(document.head);
			}
			//if there is an oncomplete method in the options, execute it when the files are loaded
			var ready = function(){
				this.fireEvent('componentLoaded', component);
				if (callback) callback();
				if (options.onComplete) options.onComplete();
				//reset the loader to be this method going forward (as it's already loaded);
				this.loaders[component] = ready;
			}.bind(this);
			//if there are dependencies, load them
			if (options.require) {
				Depender.require({
					scripts: options.require,
					callback: ready
				});
			} else {
				ready();
			}
		}.bind(this);
		
		if (options.menu) CCS.Dock.addApp(component, options);
		//if this app is to autolaunch
		if (options.autolaunch) this.autolaunchers.push(this.launch.bind(this, component));
		return this;
	},

	autolaunchers: [],

	hasApp: function(component) {
		return !!this.loaders[component];
	},

	//configures a launcher (function) for a given component(string)
	setLauncher: function(component, launcher) {
		this.launchers[component] = launcher;
	},

	instances: {},

	/*
		adds an instance for z-index / focus management
		component - (string) the application name
		instance - (object) the instance of that application
	*/
	addInstance: function(component, instance){
		$(instance).store('CCS:component', component);
		if (!this.instances[component]) this.instances[component] = [];
		this.instances[component].include(instance);
		instance.addEvent('destroy', this.removeInstance.bind(this, [component, instance]));
	},
	
	/*
		given an instance of a widget, returns the component name for that widget ("FileBrowser", "JobBrowser", etc)
	*/
	getAppComponentName: function(instance) {
		var el = $(instance);
		var appName = el.retrieve('CCS:component');
		if (!appName && el != document.body) appName = this.getAppComponentName(el.getParent());
		return appName;
	},

	/*
		given the component name of a widget ("FileBrowser"), return the nice name ("File Browser")
	*/
	getAppName: function(component) {
		var appData = this.bootstraps[component];
		if (!appData) return null;
		return this.bootstraps[component].name || component;
	},

	/*
		removes an instance from z-index / focus managment
		component - (string) the application name
		instance - (object) the instance of that application
	*/
	removeInstance: function(component, instance){
		this.instances[component].erase(instance);
		return this;
	},
	/*
		brings all the windows of a specified application to the foreground
		component - (string) the application name
	*/
	focusComponent: function(component) {
		var instances = this.instances[component];
		if (!instances || !instances.length) return instances || [];
		if (instances.length == 1) {
			instances[0].enable();
		} else {
			instances.sort(function(left, right){
				var zl = $(left).getStyle('z-index').toInt();
				var zr = $(right).getStyle('z-index').toInt();
				return (zl == zr) ? 0 : (zl < zr) ? -1 : 1;
			}).each(function(instance){
				instance.bringToFront();
			});
			var last = instances.getLast();
			if (last.enable) last.enable();
		}
		return instances;
	},

	/*
		launches a given component
		component - (string) the name of the component
		args - (object) arguments to pass to its launcher
		callback - (function; optional) an optional callback passed a pointer to the instance created; passed nothing if the launch was not sucessful.
	*/
	
	launch: function(component, args, callback){
		callback = callback || $empty;
		if (!this.hasApp(component)) {
			CCS.error('Could Not Launch App', 'Sorry, we couldn\'t find an app named ' + component + '. Launching a new window instead.');
			callback();
			return window.open(args[0], component);
		}
		
		args = args ? $splat(args) : [];
		//if the component has a launcher configured
		var launch = function(){
			this.fireEvent('beforeLaunch', component);
			var launcher = this.launchers[component];
			//throw errors if we're debugging
			try {
				var launched = launcher.apply(this, args);
				if ($(launched)) $(launched).addClass('CCS-'+component.toUpperCase());
				this.addInstance(component, launched);
				var after = function(){
					this.fireEvent('afterLaunch', component);
					launched.removeEvent('afterLaunch', after);
				}.bind(this);
				launched.addEvent('load', after);
				callback(launched);
				return launched;
			} catch(e) {
				callback();
				if (dbug.enabled) {
					dbug.error('could not launch ' + component + ': ', e);
					launcher.apply(this, args);
				} else {
					alert('Sorry, there was an error launching ' + component + '.\n Try again, or contact us for help.');
				}
			}
		}.bind(this);
		if (this.hasLoaded(component)) {
			return launch();
		} else {
			//add an event so it launches after it loads
			var loaded;
			this.addEvent('componentLoaded', function(comp){
				if (loaded) return;
				if (comp == component) {
					loaded = true;
					return launch();
				}
			}.bind(this));
		}
		this.load(component);
	},

	//load a list of components, e.g.
	//.load("FileBrowser", "JobTracker", "Login", etc)
	//callback - (function) method to execute when the component loads
	load: function(component, callback) {
		callback = callback || $empty;
		if (!this.hasLoaded(component)) {
			this.fireEvent('beforeLoad', component);
			this.loaders[component](callback);
		} else {
			this.fireEvent('componentLoaded', component);
			callback();
		}
		return this;
	},

	hasLoaded: function(component) {
		return !!this.loadedComponents[component];
	},

	//iterates over all the open windows and serializes their state for restoration
	//returns: an object of component, size, position, url, and options for each open window
	serialize: function(){
		var state = {};
		//loop through all the components for which there are instances
		$each(this.instances, function(instances, component) {
			if (!state[component]) state[component] = [];
			//loop through each instance for the component (all the windows)
			instances.each(function(instance) {
				//if it has a serialize method and it's not destroyed
				//store its serialization
				if (instance.serialize && !instance.destroyed) state[component].push(instance.serialize());
			});
		});
		//return the object of all the states
		return state;
	},

	//restores a state to the desktop
	//states - (object) the state of all the open apps (returned by .serialize())
	restore: function(states) {
		var hidden, msg;
		//hider obscures the desktop while it loads apps
		var hider = function(){
			//if the msg is already displayed/present, then exit
			if (msg) return;
			hidden = true;
			//hide the desktop
			$('ccs-desktop').setStyle('opacity', 0);
			//create a holder for the messaging, inject it, and center it
			msg = new Element('div', {
				'class':'loadingmsg',
				html: '<p>Restoring your session...</p>'
			}).inject(document.body).adopt(new Element('a', {
				text: 'Clear your session',
				events: {
					click: function(){
						this.resetAndRefresh();
					}.bind(this)
				}
			})).position();
		}.bind(this);
		//fades the message out and restores the desktop
		var fader = function(){
			msg.fade('out').get('tween').chain(function(){
				msg.destroy();
			});
			$('ccs-desktop').fade('in');
		};
		//reference count the number of components we're going to load
		var components_to_load = 0;
		var loaded_component;
		//iterate over all the components
		$each(states, function(componentStates, component) {
			//if there are no states in this app (component) to load, exit
			if (!componentStates.length) return;
			loaded_component = true;

			//hide the desktop
			hider();
			//increment the component counter
			components_to_load++;

			//define a launcher function for this particular component
			var launcher = function(){
				//update the message to the user
				msg.set('html', 'Restoring ' + this.getBootstrap(component).name + '...');
				//decrement the reference counter
				components_to_load = components_to_load - 1;
				//for each app that was open, launch it
				componentStates.each(function(state) {
					var instance = this.launch(component, [state.path, state.options]).restore(state);
				}, this);
				//if there are no more components, fade in the desktop
				if (components_to_load == 0) {
					this.pollSession();
					fader.delay(500);
				}
			}.bind(this);

			//if the component is already loaded, call the launcher
			if (this.hasLoaded(component)) {
				launcher();
			} else {
				//else add an event to call the launcher when the component is loaded
				var launched;
				this.addEvent('componentLoaded', function(comp){
					if (comp == component && !launched) {
						launched = true;
						launcher();
					}
				});
				//load the component
				this.load(component);
			}
		}, this);
		if (!loaded_component) this.pollSession();
		return loaded_component;
	},

	//returns true if there is a state to restore
	isClearSession: function(){
		return new URI().getData('clearSession') == "true";
	},

	//stores the current desktop state
	store: function(){
		if (this.noSession) return;
		var hashString = JSON.encode(this.serialize());
		var jsonRequest = new Request.JSON({
				url: this.stateUrl, 
				method: "post", 
				onFailure:function() {
					$clear(CCS.Desktop.store_periodical);
				}
		});
		jsonRequest.send("set=" + hashString);
	},

	//delete the user session
	resetSession: function(){
		var jsonRequest = new Request.JSON({
			url: this.stateUrl,
			method:"post",
			async: false
		});
		jsonRequest.send("delete");
	},

	resetAndRefresh: function(){
		this.resetSession();
		document.body.hide();
		this.noSession = true;
		window.location.href = "/";
	},

	//restores the desktop
	restoreDesktop: function(){
		if (this.isClearSession()) {
			this.resetAndRefresh();
			return;
		}
		var state = this.getState();
		return state ? this.restore(JSON.decode(state)) : false;
	},

	//This request is synchronous because this method determines whether or not there is a session to restore.  It is a decision point, at which the app continues in one of two ways.
	//Either restoration, or be initializing the default desktop.  In order to do this synchronously we would need to use callbacks.
	getState: function() {
		var result;
		var jsonRequest = new Request.JSON({
			url: this.stateUrl,
			async: false,
			method: "post",
			onSuccess: function(data){
				result = data;
			}
		});
		jsonRequest.send();
		return result;
	},

	pollSession: function(){
		//store the state of the desktop every 10 seconds
		CCS.User.withUser(function(){
			CCS.Desktop.store_periodical = CCS.Desktop.store.periodical(30000, CCS.Desktop);
		});
	}
	
};

//store the state of the desktop on unload
window.addEvent('unload', function(){
	CCS.Desktop.store();
});

$extend(CCS.Desktop, new Events);
$extend(CCS.Desktop, new Options);
