// Copyright (c) 2010, Cloudera, Inc. All rights reserved.
/*
---
description: Dock UI Component for Desktop
provides: [CCS.Dock]
requires: [ccs-shared/CCS, clientcide/StickyWin.PointyTip, /CCS.User]
script: CCS.Dock.js

...
*/
CCS.Dock = {
	initialize: function(){
		this.element = $('ccs-dock');
		this.statusContent = $('ccs-dock-status-content');
		//show/hide tips when mousing over dock icons; launch the app when clicked
		this.icons = $('ccs-dock-icons').addEvents({
			'mouseover:relay(a)': this.showAppTip.bind(this),
			'mouseout:relay(a)': this.hideAppTip.bind(this),
			'click:relay(a)': this.launchApp.bind(this),
			'dblclick:relay(a)': function(e, a){
				this.launchApp(e, a, true);
			}.bind(this)
		});
		//an instance of PointyTip for the application names
		this.tip = new StickyWin.PointyTip({
			point: 6,
			destroyOnClose: false,
			showNow: false,
			pointyOptions: {
				closeButton: false
			},
			offset: {
				y: -6
			}
		});
		$(this.tip).addClass('dock-tip');
	},
	/*
		shows the tip popup when an application is moused over
		e - the mouseover event
		a - the link to the app
	*/
	showAppTip: function(e, a){
		var component = a.retrieve('CCS-app');
		if (!component) return;
		var name = CCS.Desktop.getBootstrap(component).name;
		this.tip.pointy.setContent(name);
		this.tip.show();
		this.tip.position({
			relativeTo: a
		});
	},
	/*
		hides the app tip
		e - the mouseout event
		a - the link to the app
	*/
	hideAppTip: function(e, a){
		this.tip.hide();
	},
	apps: {},
	/*
		registers an app on the doc; builds its icon
		component - (string) the application name
		appData - (object) the metadata from the bootstrap
	*/
	addApp: function(component){
		var appData = CCS.Desktop.getBootstrap(component);
		if (!appData.menu || !appData.menu.img) return;
		var lnk = new Element('a', {
			id: appData.menu.id
		});
		var img = new Element('img', appData.menu.img).addClass('ccs-icon').inject(lnk);
		var dock = $('ccs-dock-icons');
		lnk.inject(dock);
		// Sort the menu.
		var sortkey = function(a_left, a_right) {
			var left = a_left.getElement('img').get('title');
			var right = a_right.getElement('img').get('title');
			return (left == right) ? 0 : (left < right) ? -1 : 1;
		};
		dock.getChildren().sort(sortkey).inject(dock);
		lnk.store('CCS-app', component);
	},
	/*
		launches an app; if the app is open already, brings its windows to the foreground
		TODO: manage creating new windows for an existing open app
		e - (event) the click event
		a - (element) the app link
		launchAnother - (boolean) if true (defaults to false), launch another copy of the app even if one is open
	*/
	launchApp: function(e, a, launchAnother) {
		e.preventDefault();
		//get the component from the link
		var component = a.retrieve('CCS-app');
		//if we're in the middle of launching that app *from the dock*, stop
		if (this.launching.contains(component)) return;
		//get the instances for this app and focus them
		var instances = CCS.Desktop.focusComponent(component);
		//if there are no instances, or if we're explicitly launching another one (double click), launch it
		if (!instances.length || launchAnother) {
			//store that we're launching this
			this.launching.push(component);
			var launched;
			//add a timeout in case we fail to get a callback
			(function(){
				//if we haven't launched after 10 sec, drop the reference so the user can try again
				if (!launched) this.launching.erase(component);
				dbug.log('removing dock launcher reference counter for %s launch request; no answer in 10 seconds', component);
			}).delay(10000, this);
			//launch it
			CCS.Desktop.launch(component, a.get('href'), function(){
				//flag that this succeeded (even if the app failed to display, we can drop our reference counter)
				launched = true;
				//remove that we're storing this
				this.launching.erase(component);
			}.bind(this));
		}
	},
	launching: []
};
