# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#!/usr/bin/env python
"""
Tests for filebrowser views
"""
from nose.plugins.attrib import attr
from hadoop import mini_cluster
from desktop.lib.django_test_util import make_logged_in_client
from nose.tools import assert_true, assert_false, assert_equal
import logging

LOG = logging.getLogger(__name__)

@attr('requires_hadoop')
def test_listdir():
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = make_logged_in_client()
    cluster.fs.setuser(cluster.superuser)
    cluster.fs.mkdir('/test-filebrowser/listdir')
    response = c.get('/filebrowser/view/test-filebrowser/')
    paths = [f['path'] for f in response.context['files']]
    assert_true("/test-filebrowser/listdir" in paths)
  finally:
    cluster.shutdown()


@attr('requires_hadoop')
def test_view():
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = make_logged_in_client()
    cluster.fs.setuser(cluster.superuser)

    cluster.fs.mkdir('/test-filebrowser/')
    f = cluster.fs.open('/test-filebrowser/test-view', "w")
    f.write("hello")
    f.close()
    response = c.get('/filebrowser/view/test-filebrowser/test-view')
    assert_equal(response.context['view']['contents'], "hello")
  finally:
    cluster.shutdown()


@attr('requires_hadoop')
def test_edit():
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    c = make_logged_in_client(cluster.superuser)
    cluster.fs.setuser(cluster.superuser)

    cluster.fs.mkdir('/test-filebrowser/')
    # File doesn't exist - should be empty
    test_path = '//test-filebrowser//test-edit'
    # (this path is non-normalized to test normalization too)
    edit_url = '/filebrowser/edit' + test_path
    response = c.get(edit_url)
    assert_equal(response.context['form'].data['path'],
                 test_path)
    assert_equal(response.context['form'].data['contents'], "")

    # Just going to the edit page and not hitting save should not
    # create the file
    assert_false(cluster.fs.exists(test_path))

    # Put some data in there and post
    new_contents = "hello world from editor"
    response = c.post("/filebrowser/save", dict(
        path=test_path,
        contents=new_contents), follow=True)
    assert_equal(response.context['form'].data['path'],
                 test_path)
    assert_equal(response.context['form'].data['contents'],
                 new_contents)

    # File should now exist
    assert_true(cluster.fs.exists(test_path))
    # And its contents should be what we expect
    f = cluster.fs.open(test_path)
    assert_equal(f.read(), new_contents)
    f.close()

    # We should be able to overwrite the file with another save
    new_contents = "hello world again from editor"
    response = c.post("/filebrowser/save", dict(
        path=test_path,
        contents=new_contents), follow=True)
    assert_equal(response.context['form'].data['path'],
                 test_path)
    assert_equal(response.context['form'].data['contents'],
                 new_contents)
    f = cluster.fs.open(test_path)
    assert_equal(f.read(), new_contents)
    f.close()

    # TODO(todd) add test for maintaining ownership/permissions
  finally:
    cluster.shutdown()
