#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

from desktop.lib.django_util import render_json, render

from django.contrib.auth import authenticate, login, get_backends
from desktop.auth.backend import AllowFirstUserDjangoBackend

def login_ajax(request):
  user = authenticate(username=request.POST.get('username'),
                      password=request.POST.get('password'))
  if user:
    login(request, user)

    return render_json(dict(
        success=True,
        user_data=_profile_dict(user)))
  else:
    return render_json(dict(
        success=False))
login_ajax.login_notrequired=True

def login_form(request):
  backends = get_backends()
  first_login_ever = False
  for be in backends:
    if isinstance(be, AllowFirstUserDjangoBackend) and be.is_first_login_ever():
      first_login_ever = True

  return render("login.html", request,
                { 'first_login_ever': first_login_ever },
                force_template=True)
login_form.login_notrequired = True

def profile(request):
  """
  Dumps JSON for user-profile information.
  """
  return render(None, request, _profile_dict(request.user))


def get_login_count(request):
  """
  Increment and return the number of times the user has logged in.
  This is called at desktop load-time to decide whether to show
  the welcome message.
  """
  # TODO(todd) is there an atomic increment option in this ORM?
  profile = request.user.get_profile()
  profile.login_count += 1
  profile.save()
  return render(None, request, profile.login_count)


def _profile_dict(user):
  return dict(
    username=user.username,
    first_name=user.first_name,
    last_name=user.last_name,
    last_login=str(user.last_login), # datetime object needs to be converted
    profile=user.get_profile(),
    email=user.email)

