#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Tests for the flash messaging support

from desktop.middleware import FlashMessenger
from desktop.lib.django_test_util import make_logged_in_client

from nose.tools import assert_true, assert_equal
import simplejson
import UserDict

class MockRequest(object):
  def __init__(self):
    self.session = UserDict.UserDict()

def minimal_flow_test():
  r = MockRequest()
  m = FlashMessenger(r)
  output = m.get()
  assert_true(len(output) == 0)

def message_test():
  r = MockRequest()
  m = FlashMessenger(r)
  m.put("Hi")
  m.put("Bye")
  assert_equal(2, len(r.session["flashMessages"]))
  assert_equal(True, r.session.modified)
  output = m.get()
  assert_equal(None, r.session.get("flashMessages"))
  assert_equal(["Hi", "Bye"], output)
  
def cross_request_flash_messages_test():
  c = make_logged_in_client()
  assert_equal(None, c.session.get('flashMessages'))
  c.get('/jframegallery/flash.html')
  assert_equal(3, len(c.session['flashMessages']))
  response = c.get('/jframegallery/', dict(format="embed"))
  assert_equal(3, len(simplejson.loads(response["X-Cloudera-Flash-Messages"])))
  assert_equal(None, c.session.get('flashMessages'))
  response = c.get('/jframegallery/', dict(format="embed"))
  assert_equal(None, response.get("X-Cloudera-Flash-Messages", None))
  
def missing_slash_test():
  c = make_logged_in_client()
  response = c.get('/jframegallery')
  assert_equal(301, response.status_code) # redirect
