#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

from hadoop.cluster import get_all_hdfs
from desktop import conf
from hadoop.fs import LocalSubFileSystem
from desktop.lib.apputil import has_hadoop

_filesystems = None

def _init_filesystems():
  """Initialize the module-scoped filesystem dictionary."""
  global _filesystems
  if _filesystems is not None:
    return
  _filesystems = {}

  if has_hadoop():
    # Load HDFSes
    _filesystems.update(get_all_hdfs())

  # Load local
  for identifier in conf.LOCAL_FILESYSTEMS.keys():
    local_fs = LocalSubFileSystem(
        conf.LOCAL_FILESYSTEMS[identifier].PATH.get())
    if identifier in _filesystems: 
      raise Exception(("Filesystem '%s' configured twice. First is " +
        "%s, second is local FS %s") % (identifier, _filesystems[identifier], local_fs))
    _filesystems[identifier] = local_fs

def get_filesystem(name):
  """Return the filesystem with the given name. If the filesystem is not defined,
  raises KeyError"""
  _init_filesystems()
  return _filesystems[name]


def reset():
  """
  reset() -- Forget all cached filesystems and go to a pristine state.
  """
  global _filesystems
  _filesystems = None
