#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Path-related utilities
#
# These are useful for testing, but shouldn't be heavily depended on,
# since paths have a tendency to change during packaging.

import os

"""
The project directory structure:
    root/                               <-- Project root (build root, run root)
        desktop/                        <-- Desktop root
            apps/...
            core/
              src/desktop/lib/paths.py  <-- You're reading this file
              ...
            core-apps/...
        ext/
            thirdparty/...
"""


def __get_root(*append):
  """
  Returns the root directory of the project.
  """
  if append is None:
    append = [ ]
  path = os.path.join(
              os.path.dirname(__file__),
              "..", "..", "..", "..", "..",
              *append)
  return os.path.abspath(path)


def get_build_dir(*append):
  """
  Returns 'build' directory for Desktop.

  This is used for temporary (and testing) artifacts.
  This is not the root source path.
  """
  return __get_root(*append)


def get_desktop_root(*append):
  """
  Returns the directory for Desktop.
  """
  return __get_root("desktop", *append)


def get_thirdparty_root(*append):
  """
  Returns the ext/thirdparty directory.
  """
  return __get_root("ext", "thirdparty", *append)


def get_run_root(*append):
  """
  Returns the run time root directory
  """
  return __get_root(*append)
