#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

from desktop.lib import apputil
import os
from django.template.loaders import app_directories
import logging
import sys
from django.template import TemplateDoesNotExist

# Django is bad at namespacing their templates,
# and auth's templates live in the admin module.
APP_REMAP = {
  "django.contrib.auth": "django.contrib.admin"
}

def load_template_source(name, dirs=[]):
  app = apputil.get_current_app()
  if not app:
    logging.info("No app found while loading template %s" % name)
    raise TemplateDoesNotExist("No app: " + name)
  if app in APP_REMAP:
    app = APP_REMAP[app]
  # We can import the app, and then find it in sys.modules,
  # thereby, in cases of "a.b.c", finding the whole thing, and not
  # just "a".
  __import__(app)
  app_module = sys.modules[app]
  app_dir = os.path.dirname(app_module.__file__)
  default_dir = os.path.join(app_dir, 'templates')

  return app_directories.load_template_source(name, [default_dir])

load_template_source.is_usable = True
