#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""Utilities for views (text and number formatting, etc)"""

import math
import datetime

def big_filesizeformat(bytes):
  if bytes is None or bytes is "":
    return "N/A"

  assert bytes >= 0

  # Special case small numbers (including 0), because they're exact.
  if bytes < 1024:
    return "%d B" % bytes

  units = ["B", "KB", "MB", "GB", "TB", "PB"]
  index = int(math.floor(math.log(bytes, 1024)))
  index = min(len(units) - 1, index)

  return( "%.1f %s" % (bytes / math.pow(1024, index), units[index]) )

def format_time_diff(start=None, end=None):
  """
    formats the difference between two times as Xd:Xh:Xm:Xs
  """
  if (end is None):
    end = datetime.datetime.now()
  diff = end - start
  minutes, seconds = divmod(diff.seconds, 60)
  hours, minutes = divmod(minutes, 60)
  days = diff.days
  output = []
  written = False
  if days:
    written = True
    output.append("%dd" % days)
  if written or hours:
    written = True
    output.append("%dh" % hours)
  if written or minutes:
    output.append("%dm" % minutes)
  output.append("%ds" % seconds)
  return ":".join(output)
