#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Dumps configuration data for debugging.

Note that this dump representation is not machine readable;
this command is still a few steps away from dumping a ConfigObj-compatible,
textual representation.
"""
from django.core.management.base import NoArgsCommand
import desktop.appmanager
import textwrap

from desktop.lib.conf import BoundContainer, is_anonymous

class Command(NoArgsCommand):
  def __init__(self, *args, **kwargs):
    super(Command, self).__init__(*args, **kwargs)
    self.indent = 0

  """Prints documentation for configuration."""
  def handle_noargs(self, **options):
    print "Dumping configuration..."
    print
    self.recurse(desktop.lib.conf.GLOBAL_CONFIG)

  def p(self, s):
    print " "*self.indent + s

  def fill(self, s):
    print textwrap.fill(s.strip(),
      initial_indent=" "*self.indent, subsequent_indent=" "*self.indent)
    

  def recurse(self, config_obj):
    if isinstance(config_obj, BoundContainer):
      if is_anonymous(config_obj.config.key):
        key = "/default/"
      else:
        key = config_obj.config.key

      self.p("%s:" % key)
      self.indent += 2
      print textwrap.fill(config_obj.config.help or "No help available.",
        initial_indent=" "*self.indent, subsequent_indent=" "*self.indent)
      print
      for v in config_obj.get().values():
        self.recurse(v)
      self.indent -= 2

    else:
      self.p("%s=%s" % (config_obj.config.key, config_obj.get()))
      self.indent += 2
      self.fill(config_obj.config.help or "No help available.")
      self.indent -= 2
