#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

from django.db import models
from django.contrib.auth import models as auth_models


class UserProfile(models.Model):
  user = models.ForeignKey(auth_models.User, unique=True)
  login_count = models.IntegerField(default=0, editable=True)

  def to_jsonable(self):
    return dict(login_count=self.login_count)

class UserPreferences(models.Model):
  """Holds arbitrary key/value strings."""
  user = models.ForeignKey(auth_models.User)
  key = models.CharField(max_length=20)
  value = models.TextField(max_length=4096)

# Create a user profile every time a user gets created.
def create_profile_for_user(sender, **kwargs):
  if kwargs['created']:
    p = UserProfile()
    p.user = kwargs["instance"]
    p.save()

models.signals.post_save.connect(create_profile_for_user, sender=auth_models.User)
