#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Test for RequireLoginEverywhereMiddleware in middleware.py
# 
# This test uses "nose"-style testing (no need for a TestCase),
# and nose-style assertions.

from nose.tools import *

from django.test.client import Client
import django

def test_require_login():
  c = Client()
  # We're not logged in, so expect a redirection.

  response = c.get('/profile')
  assert_true(isinstance(response, django.http.HttpResponseRedirect), "Expected redirect")
  assert_equal("http://testserver/accounts/login/?next=/profile", response["Location"], "Expected redirection to login page")

  # AllowAllBackend should let us in.
  c.login(username="test", password="test")
  # And now we shouldn't need to be redirected.
  response = c.get('/')
  assert_equal(200, response.status_code, "Expected ok status.")


def test_ajax_require_login():
  c = Client()
  response = c.get('/profile',
                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
  assert_equal("LOGIN_REQUIRED", response["X-Cloudera-Middleware-Response"],
               "Expected magic header from middleware")
