#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

import logging
import os
import re

from django.conf.urls.defaults import patterns, include
from django.contrib import admin
from desktop import appmanager

# Set up /appname/static mappings for any apps that have static directories
def static_pattern(urlprefix, root):
  """
  First argument is the url mapping, and second argument is the
  directory to serve.
  """
  return (r'^%s/(?P<path>.*)$' % urlprefix, 'django.views.static.serve',
   { 'document_root': root, 'show_indexes': True })


admin.autodiscover()

# Some django-wide URLs
dynamic_patterns = patterns('',
  (r'^accounts/login/$', 'django.contrib.auth.views.login'),
  (r'^accounts/login_ajax/?$', 'desktop.auth.views.login_ajax'),
  (r'^accounts/logout/$', 'django.contrib.auth.views.logout', {'next_page': '/'}),
  (r'^accounts/login_count/?$', 'desktop.auth.views.get_login_count'),
  (r'^accounts/login_form/?$', 'desktop.auth.views.login_form'),
  (r'^logs$','desktop.views.log_view'),
  (r'^dump_config$','desktop.views.dump_config'),
  (r'^download_logs$','desktop.views.download_log_view'),                   
  (r'^bootstrap.js$', 'desktop.views.bootstrap'),
  (r'^profile$', 'desktop.auth.views.profile'),
  (r'^prefs/(?P<key>\w+)?$', 'desktop.views.prefs'),
  (r'^status_bar/?$', 'desktop.views.status_bar'),
  (r'^admin/', include(admin.site.urls)),
  # Top level web page!
  (r'^$', 'desktop.views.index'),
)

static_patterns = []

# Root each app at /appname if they have a "urls" module
for app in appmanager.DESKTOP_APPS:
  if app.urls:
    dynamic_patterns.extend( patterns('', ('^' + re.escape(app.name) + '/',
                                           include(app.urls))) )
    app.urls_imported = True

  # Root a /appname/static if they have a static dir
  if app.static_dir:
    static_patterns.append(
      static_pattern('%s/static' % app.name, app.static_dir))

# TODO this stuff should probably be moved into a "ui" lib or somesuch so it
# is autodiscovered
def buildpath(d):
  return os.path.join(os.path.dirname(__file__), "..", '..', '..', d)
static_patterns.append(static_pattern("static", buildpath("core/static")))

urlpatterns = patterns('', *static_patterns) + dynamic_patterns

for x in dynamic_patterns:
  logging.debug("Dynamic pattern: %s" % x)
for x in static_patterns:
  logging.debug("Static pattern: %s" % str(x))
