// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: A JBrowser is a window that encapsulates a JFrame and a linked HistoryMenu.
provides: [CCS.JBrowser]
requires: [/CCS.JFrame, art-widgets/ART.Browser, art-widgets/ART.SolidWindow, /CCS.JFrame.AutoRefresh, /CCS.JFrame.Collapsible, /CCS.JFrame.DoubleClickDelegate, /CCS.JFrame.HtmlTable, /CCS.JFrame.OverText, /CCS.JFrame.SubmitOnChange, /CCS.JFrame.Clearfix, /CCS.JFrame.ContextMenu, /CCS.JFrame.FormRequest, /CCS.JFrame.NoOverflow, /CCS.JFrame.SplitView, /CCS.JFrame.Tabs, /CCS.JFrame.ConfirmAndPost, /CCS.JFrame.PromptAndPost, /CCS.JFrame.Refresh, /CCS.JFrame.Target, /CCS.JFrame.Alert, /CCS.JFrame.Prompt, /CCS.JFrame.ArtButtons ]
script: CCS.JBrowser.js

...
*/
(function(){

	var jbrowser = {

		options: {
			//the onLoad event fires when new content loads
			//onLoad: $empth(view),
			
			//display the history widget in the header?
			displayHistory: true,
			//width: default width
			width: 800,
			//showNow: if true, the window is displayed on instantiation
			showNow: false,
			//draggable: if true, the window can be dragged around
			draggable: true,
			//windowTitler: passed the default title from the response, returns a title for the window.
			windowTitler: function(title) {
				return title || this.options.windowOptions.caption || '';
			},
			jframeOptions: {}
		},

		initialize: function(path, options) {
			options = options || {};
			var show = $pick(options.showNow, true);
			options.showNow = false;
			this.parent(options);
			
			this._setupHistory(path);
			if (show) {
				this.options.showNow = true;
				this.show();
			}
			this._makeJFrame(path);
			$(this).setStyle('visibility', 'hidden');
		},
		_setupHistory: function(path){
			if (this.history) {
				if (!this.options.displayHistory) {
					$(this.history).dispose();
					this.history.redraw = $empty;
				} else {
					//prevent clicks to the history element from starting the drag behavior attached to the entire header
					$(this.history).addEvent('click', function(e) { e.stopPropagation(); });
				}
				this.history.addEvents({
					refresh: function(){
						this.refresh();
					}.bind(this),
					select: function(path, title){
						if (path != this.jframe.currentPath) this.load({requestPath: path, suppressHistory: true });
					}.bind(this)
				});
			}
		},
		_makeJFrame: function(path){
			var opt = $merge({
				container: this.content,
				onLoadComplete: this._jframeLoaded.bind(this),
				onLoadError: this._jframeError.bind(this),
				size: {
					width: this.contentSize.x, 
					height: this.contentSize.y
				},
				getScroller: function(){
					return this.content;
				}.bind(this)
			}, this.options.jframeOptions);
			opt.parentWidget = this;
			this.jframe = new CCS.JFrame(path, opt);
			
			//adds mouseover/mouseout behaviors for all links that have the class
			//.frame_tip; when the user mouses over such an object, the "status"
			//area in the window footer displays the title of that element.
			this.jframe.addFilters({
				statusTips: function(container) {
					container.addEvents({
						'mouseover:relay(.frame_tip)': function(e, element){
							var tipTitle = element.retrieve('tip:title', element.get('title') || '');
							element.set('title', '');
							this.setFooter(tipTitle);
						}.bind(this),
						'mouseout:relay(.frame_tip)': function(e, element){
							this.setFooter('');
						}.bind(this)
					});
				}.bind(this)
			});

			this.toolbar = new Element('div', {
				'class':'ccs-window-toolbar',
				events: {
					mousedown: function(e){
						//prevent clicks to the toolbar element from starting the drag behavior attached to the entire header
						if (!$(e.target).hasClass('draggable'))e.stopPropagation();
					}
				}
			}).inject(this.header);
			this.jframe.applyDelegates(this.toolbar);
			this.addEvents({
				enable: function(){
					this.jframe.enable();
				}.bind(this),
				focus: function(){
					this.jframe.focus();
				}.bind(this)
			});
			this.jframe.addEvent('refresh', this._storeScroll.bind(this));
			
		},

		wait: function(start){
			start = $pick(start, true);
			if (start) this.content.spin({ fxOptions: {duration: 200} });
			else this.content.unspin();
		},

		load: function(options) {
			this.jframe.load(options);
			return this;
		},

	/*
		options are passed to jframe's renderContent method
		*/

		setContent: function(options){
			this.jframe.renderContent(options);
			return this;
		},

		refresh: function() {
			this.jframe.refresh();
			return this;
		},

		resize: function(w, h) {
			if (w == this.currentWidth && h == this.currentHeight) return;
			this.parent(w, h);
			this.jframe.resize(this.contentSize.x, this.contentSize.y);
		},

		_jframeLoaded: function(data) {
			$(this).setStyle('visibility', 'visible');
			this.setCaption(this.options.windowTitler(data.title || data.repsonsePath));
			if (data.toolbar) this.toolbar.empty().adopt(data.toolbar);
			if (!data.suppressHistory && this.history) this.history.push({ path: data.responsePath, title: data.title || data.repsonsePath});
			if (this._jframe_view != data.view) {
				if (this._jframe_view) this.contents.removeClass(this._jframe_view);
				if (data.view) {
					this.contents.addClass(data.view);
					this._jframe_view = data.view;
				}
			}
			this.fireEvent('load', data.view);
			/*
				I hate this delay, but the browser apparently needs it to render the HTML. You can't set the scroll offset of something
				thats empty (because there's no where to scroll). The duration may require some additional care with diff. browsers or
				slower computers.
			*/
			this._restoreScroll.delay(50, this);
		},

		_jframeError: function(error) {
			if (this.refreshButton) this.refreshButton.unspin();
		},

		//returns an object with the dimensions, location, path, and options
		serialize: function(){
			return {
				styles: $(this).getStyles(['top', 'left']),
				size: this.getSize(),
				path: this.jframe.currentPath,
				options: this.options
			};
		},

		//restore's a window to a given location and size
		restore: function(state){
			//restore the position
			$(this).setStyles(state.styles);
			//restore the size
			this.resize(state.size.width, state.size.height);
			return this;
		},
		
		minMax: function(operation){
			if (operation == "maximize") {
				var beforeStr = 'before'+ operation.capitalize();
				if (this[beforeStr]) {
					this.element.setStyles(this.posBefore);
				} else {
					this.posBefore = this.element.getStyles('top', 'left');
					this.element.setStyles({
						top: 0,
						left: 0
					});
				}
			}
			this.parent(operation);
		},

		//returns the elements whose scroll offset we want to store
		//this includes any element with the .save_scroll class
		//and also the contents of the window itself.
		_getScrollElements: function(){
			var scrollers = $(this).getElements('.save_scroll');
			scrollers.include(this.content);
			return scrollers;
		},

		//stores the scroll offset for all the elements that we are saving
		_storeScroll: function(){
			this._scrolled = this._getScrollElements().map(function(el){
				return el.getScroll();
			});
		},

		//restores the scroll offsets to the elements we saved
		//but only if we found a matched number of each
		//note: that this behavior is only triggered on refresh. The main issue with refresh is if there's a redirect.
		_restoreScroll: function(){
			if (!this._scrolled) return;
			var scrollers = this._getScrollElements();
			if (scrollers.length == this._scrolled.length) {
				this._scrolled.each(function(data, i) {
					scrollers[i].scrollTo(data.x, data.y);
				});
			}
			this._scrolled = null;
		}

	};

	CCS.JBrowser = new Class(
		$merge({
			Extends: ART.Browser
		}, jbrowser)
	);
	CCS.JBrowser.Solid = new Class(
		$merge({
			Extends: ART.SolidWindow
		}, jbrowser)
	);
	//a window alert w/ a jframe
	CCS.JBrowser.Confirm = new Class(
		$merge({
			Extends: ART.Confirm,
			displayHistory: false
		}, jbrowser)
	);
	//shortcut for JBrowser.Confirm
	CCS.JBrowser.confirm = function(caption, content, callback, options) {
		return new CCS.JBrowser.Confirm(options.path,
			$extend(options || {}, {
				caption: caption,
				onConfirm: callback || $empty
			})
		);
	};
})();
