// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Creates a double click action for any element with the class .dbclick_delegate.
provides: [CCS.JFrame.DoubleClickDelegate]
requires: [/CCS.JFrame, /Element.Data]
script: CCS.JFrame.DoubleClickDelegate.js

...
*/

CCS.JFrame.doubleClickHandler = function(jframe, delegate){
	var getLink = function(delegate){
		//retrieve configuration from the css property JSON
		var data = delegate.get('data', 'dblclick-delegate', true);
		if (data && data.dblclick_loads) {
			//find the link that is to be activated when we double click the delegate
			//and load its contents
			return delegate.getElement(data.dblclick_loads);
		}
	};
	
	var link = getLink(delegate);
	if (link) {
		if (link.get('target')) {
			CCS.Desktop.launch(link.get('target'), link.get('href'));
		} else {
			jframe.load({
				requestPath: link.get('href')
			});
		}
	}

};

CCS.JFrame.addGlobalFilters({

	//intercept double click behaviors
	doubleClickDelegates: function(container) {

		//define our handler
		var handler = function(e, delegate){
			CCS.JFrame.doubleClickHandler(this, delegate);
		}.bind(this);
		//add this behavior to the delegate
		container.addEvent('dblclick:relay([data-dblclick-delegate])', handler);
		//remove it when we unload (incase we delegate to the jframe container)
		this.markForCleanup(function(){
			container.removeEvent('dblclick:relay([data-dblclick-delegate])', handler);
		});
	}

});