// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Creates instances of HtmlTable for any table with the css class .ccs-data_table with additional options for sortability and selectability.
provides: [CCS.JFrame.HtmlTable]
requires: [/CCS.JFrame, More/HtmlTable.Sort, More/HtmlTable.Zebra, More/HtmlTable.Select]
script: CCS.JFrame.HtmlTable.js

...
*/

CCS.JFrame.addGlobalFilters({

	htmlTable: function(container){
		//make all data tables sortable
		var selectable;
		var tables = container.getElements('table.ccs-data_table').map(function(table){
			var isSelectable = (table.hasClass('sortable') && !table.hasClass('noSelect')) || table.hasClass('selectable');
			var ht = new HtmlTable(table, {
				sortable: table.hasClass('sortable'),
				selectable: isSelectable,
				allowMultiSelect: table.hasClass('allowMultiSelect'),
				useKeyboard: false
			});
			if (isSelectable && !selectable) selectable = ht;
			return table;
		}).clean();
		//This is silly but for now, we only support keyboard shortcuts on the first
		//table in the UI. dumb, but I don't have time to hack around this.
		//TODO(nutron)
		if (selectable) {
			this.addShortcuts({
				'Select Previous Row': {
					keys: 'up',
					shortcut: 'up arrow',
					handler: function(e){
						e.preventDefault();
						selectable.shiftFocus(-1);
						if (selectable.hover) selectable.focusRow(selectable.hover);
					}.bind(this),
					description: 'Select the previous row in the table.'
				},
				'Select Next Row': {
					keys: 'down',
					shortcut: 'down arrow',
					handler: function(e){
						e.preventDefault();
						selectable.shiftFocus(1);
						if (selectable.hover) selectable.focusRow(selectable.hover);
					}.bind(this),
					description: 'Select the next row in the table.'
				}
			});
			this.markForCleanup(function(){
				this.removeShortcuts(['Select Previous Row', 'Select Next Row']);
			}.bind(this));
		}
	}

});