// Copyright (c) 2010, Cloudera, inc. All rights reserved.
/*
---
description: Creates a SplitView instances for all elements that have the css class .splitview (and children with .left_col and .right_col).
provides: [CCS.JFrame.SplitView]
requires: [/CCS.JFrame, art-widgets/ART.SplitView]
script: CCS.JFrame.SplitView.js

...
*/

CCS.JFrame.addGlobalFilters({

	splitView: function(container) {
		if (!container.get('html').contains('splitview')) return;
		container.setStyle('overflow', 'hidden');
		//for all div.splitview containers, get their left and right column and instantiate an ART.SplitView
		//if the container has the class "resizable" then make it so
		//ditto for the foldable option
		//if the left or right columns have explicit style="width: Xpx" assignments
		//resize the side to match that statement; if both have it, the right one wins
		var splitviews = container.getElements('div.splitview').map(function(splitview){
			var left = splitview.getElement('.left_col');
			var right = splitview.getElement('.right_col');
			var inlineSize = {
				left: left.style.width,
				right: right.style.width
			};
			left.setStyle('width', 'auto');
			right.setStyle('width', 'auto');
			var split = new ART.SplitView({
				resizable: splitview.hasClass("resizable"),
				foldable: splitview.hasClass("foldable"),
				parentWidget: this
			});
			split.setLeftContent(left);
			split.setRightContent(right);
			['left', 'right'].each(function(side) {
				if (inlineSize[side]) split['resize'+side.capitalize()](inlineSize[side].toInt());
				split[side].addClass('save_scroll');
			});
			var classes = splitview.get('class').split(' ');
			$(split).replaces(splitview);
			$(split).set('class', $(split).get('class') + ' ' + classes.join(' '));
			split.render();
			split.resizer = split.resize.bind(split);
			this.addEvent('resize', split.resizer);
			if (this.parentWidget && this.parentWidget.contentSize) {
				var w = this.parentWidget.contentSize.x;
				var h = this.parentWidget.contentSize.y;
				split.resizer(w, h);
			}
			return split;
		}, this);

		this.markForCleanup(function(){
			splitviews.each(function(splitview) {
				this.removeEvent('resize', splitview.resizer);
				splitview.destroy();
			}, this);
		}.bind(this));

		return splitviews;
	}

});