/*
---
description: Any JFrame response wthat has a root-level child element with the class .alert_popup is displayed in an alert overlaying the previous state which is restored when the alert is closed. If the class .jframe_renders is also applied to that element, the jframe is not restored but displays the remaining portion of the response as normal.
provides: [CCS.JFrame.Alert]
requires: [/CCS.JFrame, art-widgets/ART.Alerts]
script: CCS.JFrame.Alert.js

...
*/
// Copyright (c) 2010, Cloudera, inc. All rights reserved.
CCS.JFrame.addGlobalRenderers({

	alert: function(content, options) {
		//if the contents have an element with .alert_popup *at the root*
		//then display those contents in an alert
		var popup = content.elements.filter('.alert_popup')[0];
		if (!popup) return;
		
		//does the jframe still get to render?
		var jframe_renders = popup.hasClass('jframe_renders');

		var target = new Element('div', {'class': 'jframe_alert'});
		if (!jframe_renders) {
			//if we aren't rendering the jframe, fill the popup
			//and remove the toolbar
			this._fill(target, content);
			var toolbar = target.getElements('.toolbar');
			if (toolbar.length) toolbar.hide();
		} else {
			//otherwise we're going to leave the content object alone and clone
			//the popup, hiding the original
			this._fill(target, {
				elements: $$(popup.clone())
			});
			popup.hide();
		}

		//our method to actually show the alert
		var alerter = function(){
			var alert = this.alert(content.title, target);
			if (options) {
				alert.addEvent('destroy', function(){
					if (options.blankWindowWithError) this.getWindow().hide();
				}.bind(this));
			}
			//if jframe is rendering we remove this event which we're going to add a few lines down
			if (jframe_renders) this.removeEvent('afterRenderer', alerter);
		}.bind(this);

		//if jframe is rendering, we need to give it the thread
		//otherwise it trashes the content AFTER we position
		//which throws it off
		if (jframe_renders) this.addEvent('afterRenderer', alerter);
		else alerter();
		
		//if jframe is rendering, return false (so it will continue as if nothing happened)
		return !jframe_renders;
	}

});