#
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
PROD_INCLUDES := \
	desktop/ \
	virtualenv_support \
	ext/thirdparty \
	example-hadoop-confs \
	LICENSE.txt \
	virtual-bootstrap.py \
	VERSION_DATA

PROD_EXCLUDES := \
	desktop/apps/jframegallery \
	desktop/apps/beeswax \
	desktop/conf \
	desktop/desktop.db \
	desktop/apps/hello \
	desktop/apps/visualizer \
	desktop/devtools.mk \
	ext/thirdparty/README.md \
	ext/thirdparty/js/manifest.json

# Macro to remove things we don't want to package
define remove_devtree_exclusions
	@find $(1) \( -name '.git' -o \
		      -name '.gitignore' -o \
		      -name '.*~' -o \
		      -name '*.py[co]' -o \
		      -name '.*.sw?' \) -prune -exec rm -rf {} \;
endef

# <<<< CLOUDERA ONLY

###################################
# Packaging for SDK release
###################################

# Make a tarball
.PHONY: sdk
sdk: $(BLD_DIR_SDK)
	@tar -C $(BLD_DIR_SDK)/.. -czf $(BLD_DIR_SDK_TGZ) cloudera-desktop-SDK-$(VERSION)
	@echo "--- Generated $(BLD_DIR_SDK_TGZ)"

.PHONY: sdkdocs
sdkdocs: $(BLD_DIR_SDK)/docs

$(BLD_DIR_SDK)/docs: docs/sdk/sdk.md
	@mkdir -p $@
	cp docs/sdk/*.png $@
	cp docs/docbook.css $@
	@# This is somewhat heinous, but works fine and avoids the need
	@# to install markdown on developer systems, or depend on building
	@# out the virtual env to build sdk docs
	PYTHONPATH=desktop/core/ext-py/markdown \
	  desktop/core/ext-py/markdown/bin/markdown \
	  docs/sdk/sdk.md -x toc > $@/sdk.html
	  @echo "--- Generated $@/sdk.html"


.PHONY: docs
docs: $(BLD_DIR_SDK)/docs

.PHONY: web
web: $(BLD_DIR_WEB)

$(BLD_DIR_WEB): docs/*.txt $(BLD_DIR_SDK)/docs
	@echo "Compiling docs for web site"
	(cd docs && ./build_site.sh)
	@mkdir -p $@/sdk
	@cp -rv docs/output/* $@
	@cp -rv $(BLD_DIR_SDK)/docs/* $@/sdk

SDK_INCLUDES := $(PROD_INCLUDES) \
	tarballs.mk \
	packaging.mk \
	tools/scripts/create_virtual_bootstrap.py \
	tools/scripts/configure-hadoop.sh

SDK_EXCLUDES := \
	desktop/conf \
	ext/thirdparty/README.md \
	ext/thirdparty/js/manifest.json \
	desktop/apps/beeswax


#
# TODO(bc): There are a lot of direct references here. Too painful to fix.
# TODO(todd): other .mk files
#
.PHONY: $(BLD_DIR_SDK)
$(BLD_DIR_SDK): clean crepo sdkdocs
	@echo "--- Preparing SDK tree at $@"
	@rm -rf $@
	@mkdir -p $@
	tar -cf - $(SDK_INCLUDES) | tar -C $(BLD_DIR_SDK) -xf -
	@echo "---- Removing exclusions"
	@for i in $(SDK_EXCLUDES) ; do rm -rf $(BLD_DIR_SDK)/$$i ; done
	$(call remove_devtree_exclusions,$(BLD_DIR_SDK))
	@echo "---- Copying misc files"
	@cp $(ROOT)/dist/* $(BLD_DIR_SDK)
	@mv $(BLD_DIR_SDK)/desktop/conf{.dist,}
	@echo "---- Stripping Makefiles"

	@$(call STRIP_CLOUDERA, $(DESKTOP_DIR)/Makefile, $(BLD_DIR_SDK)/desktop/Makefile)
	@$(call STRIP_CLOUDERA, $(DESKTOP_DIR)/devtools.mk, $(BLD_DIR_SDK)/desktop/devtools.mk)
	@$(call STRIP_CLOUDERA, $(ROOT)/Makefile, $(BLD_DIR_SDK)/Makefile)
	@$(call STRIP_CLOUDERA, $(ROOT)/Makefile.vars, $(BLD_DIR_SDK)/Makefile.vars)
# END CLOUDERA ONLY >>>>

###################################
# Packaging for production release
###################################

# Make a tarball
.PHONY: prod
prod: $(BLD_DIR_PROD_TGZ)

$(BLD_DIR_PROD_TGZ): $(BLD_DIR_PROD)
	@tar -C $(BLD_DIR_PROD)/.. -czf $(BLD_DIR_PROD_TGZ) cloudera-desktop-$(VERSION)
	@echo "--- Generated $(BLD_DIR_PROD_TGZ)"


#
# TODO(bc): There are a lot of direct references here. Too painful to fix.
#
.PHONY: $(BLD_DIR_PROD)
$(BLD_DIR_PROD): clean crepo
	@echo "--- Preparing general distribution tree at $@"
	@rm -rf $@
	@mkdir -p $@

	tar -cf - $(PROD_INCLUDES) | tar -C $(BLD_DIR_PROD) -xf -
	@echo "---- Removing exclusions"
	@for i in $(PROD_EXCLUDES) ; do rm -rf $(BLD_DIR_PROD)/$$i ; done
	$(call remove_devtree_exclusions,$(BLD_DIR_PROD))
	@echo "---- Copying misc files"
	@cp $(ROOT)/dist/* $(BLD_DIR_PROD)
	@mv $(BLD_DIR_PROD)/desktop/conf{.dist,}
	@echo "---- Stripping Makefiles"
	@$(call STRIP_SDK, $(DESKTOP_DIR)/Makefile, $(BLD_DIR_PROD)/desktop/Makefile)
	@$(call STRIP_SDK, $(ROOT)/Makefile, $(BLD_DIR_PROD)/Makefile)
	@$(call STRIP_SDK, $(ROOT)/Makefile.vars, $(BLD_DIR_PROD)/Makefile.vars)
	cd $(BLD_DIR_PROD) && make clean
